# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import subprocess
import logging

logger = logging.getLogger(__name__)


def get_app_config():
    try:
        import settings
        return {
            'HIGHCHARTS_SERVER_HOST': getattr(settings, 'HIGHCHARTS_SERVER_HOST', 'localhost'),
            'HIGHCHARTS_SERVER_PORT': getattr(settings, 'HIGHCHARTS_SERVER_PORT', '6060'),
            'HIGHCHARTS_SERVER_WORKERS': getattr(settings, 'HIGHCHARTS_SERVER_WORKERS', None),
            'HIGHCHARTS_SERVER_WORK_LIMIT': getattr(settings, 'HIGHCHARTS_SERVER_WORK_LIMIT', None),
            'HIGHCHARTS_SERVER_LOG_LEVEL': getattr(settings, 'HIGHCHARTS_SERVER_LOG_LEVEL', None),
            'HIGHCHARTS_SERVER_QUEUE_SIZE': getattr(settings, 'HIGHCHARTS_SERVER_QUEUE_SIZE', None),
            'HIGHCHARTS_SERVER_RATE_LIMIT': getattr(settings, 'HIGHCHARTS_SERVER_RATE_LIMIT', None)
        }
    except ImportError:
        return {
            'HIGHCHARTS_SERVER_HOST': 'localhost',
            'HIGHCHARTS_SERVER_PORT': '6060'
        }


def run_server():
    logger.info('Starting Highcharts Export Server')

    config = get_app_config()
    try:
        args = [
            "highcharts-export-server",
            "--enableServer", "1",
            "--host", config.get('HIGHCHARTS_SERVER_HOST', 'localhost'),
            "--port", config.get('HIGHCHARTS_SERVER_PORT', '6060'),
            "--nologo", "1"
        ]

        if config.get('HIGHCHARTS_SERVER_WORKERS'):
            args.extend(["--workers", config.get('HIGHCHARTS_SERVER_WORKERS')])

        if config.get('HIGHCHARTS_SERVER_WORK_LIMIT'):
            args.extend(["--workLimit", config.get('HIGHCHARTS_SERVER_WORK_LIMIT')])

        if config.get('HIGHCHARTS_SERVER_LOG_LEVEL'):
            args.extend(["--logLevel", config.get('HIGHCHARTS_SERVER_LOG_LEVEL')])

        if config.get('HIGHCHARTS_SERVER_QUEUE_SIZE'):
            args.extend(["--queueSize", config.get('HIGHCHARTS_SERVER_QUEUE_SIZE')])

        if config.get('HIGHCHARTS_SERVER_RATE_LIMIT', False):
            args.extend(["--rateLimit", config.get('HIGHCHARTS_SERVER_RATE_LIMIT')])

        subprocess.run(args)
    except FileNotFoundError as e:
        logger.error('Failed to run the Highcharts Export Server: {}'.format(e))
    except KeyboardInterrupt:
        pass


if __name__ == '__main__':
    run_server()



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from .content_publishing_report import ContentPublishingReportResource, \
    ContentPublishingReportService

from analytics.common import register_report


def init_app(app):
    endpoint_name = 'content_publishing_report'
    service = ContentPublishingReportService(endpoint_name, backend=superdesk.get_backend())
    ContentPublishingReportResource(endpoint_name, app=app, service=service)
    register_report('content_publishing_report', 'content_publishing_report')

    superdesk.privilege(
        name='content_publishing_report',
        label='Analytics - Content Publishing Reports',
        description='User can view Content Publishing Reports'
    )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from .axis import Axis

from copy import deepcopy


class Chart:
    """Class instance for generating a config for use with Highcharts"""

    def __init__(self, chart_id, **kwargs):
        """Initialise the data for the chart config

        kwargs:
            - ``chart_id (str)``: the id to be given to the chart
            - ``title (str)``: the title of the chart
            - ``subtitle (str)``:  The subtitle of the chart
            - ``start_of_week (int)``: The starting day of the week (0=Sunday, 6=Saturday)
            - ``timezone_offset (int)``:  The UTC offset in minutes for the timezone to use
            - ``use_utc (bool=True)``:  Use UTC in the datetime fields
            - ``height (int)``:  The height of the chart
            - ``legend_title (str)``: The title for the legend
            - ``tooltip_header (str)``: The tooltip header format
            - ``tooltip_pointer (str)``: The tooltip point format
            - ``data_labels (bool=False)``: Enable/Disable data labels
            - ``data_label_format (str)``: Data label format
            - ``colour_by_point (bool=False)``: One colour per series or one colour per point
            - ``full_height (bool=False)``: Forces the chart to render full height
            - ``default_config (dict={})``: Default config options for this chart
            - ``zoom_type (str)``: The zoom type applied to highcharts
            - ``translations (dict={})``: Field name & values translations
        """

        self.id = chart_id
        self.chart_type = 'highcharts'
        self.use_utc = True
        self.data_labels = False
        self.full_height = False

        self.axis = []
        self.config = {}
        self.translations = {}
        self.default_config = {}

        self.title = None
        self.subtitle = None
        self.start_of_week = None
        self.timezone_offset = None
        self.height = None
        self.tooltip_header = None
        self.tooltip_point = None
        self.legend_title = None
        self.colour_by_point = None
        self.zoom_type = None
        self.data_label_format = None

        self.set_options(**kwargs)

    def set_options(self, **kwargs):
        """Sets the options for the chart"""

        for key, value in kwargs.items():
            self.__dict__[key] = value

        return self

    def get_title(self):
        """Returns the title string to use for the chart"""

        return self.title

    def gen_title_config(self, config):
        """Sets the title config to use for the chart"""

        title = self.get_title()

        if title is not None:
            if 'title' not in config:
                config['title'] = {}
            config['title']['text'] = title

        return config

    def get_subtitle(self):
        """Returns the subtitle string to use for the chart"""

        return self.subtitle

    def gen_subtitle_config(self, config):
        """Sets the subtitle config to use for the chart"""

        subtitle = self.get_subtitle()

        if subtitle is not None:
            if 'subtitle' not in config:
                config['subtitle'] = {}
            config['subtitle']['text'] = subtitle

        return config

    def gen_legend_config(self, config):
        """Sets the legend config to use for the chart"""

        if 'legend' not in config:
            config['legend'] = {}

        if self.legend_title is None:
            config['legend']['enabled'] = False
        else:
            config['legend']['enabled'] = True
            config['legend']['title'] = {'text': self.legend_title}

        return config

    def gen_tooltip_config(self, config):
        """Sets the tooltip config to use for the chart"""

        if 'tooltip' not in config:
            config['tooltip'] = {}

        if self.tooltip_header is not None:
            config['tooltip']['headerFormat'] = self.tooltip_header

        if self.tooltip_point is not None:
            config['tooltip']['pointFormat'] = self.tooltip_point

        return config

    def gen_plot_config(self, config):
        """Sets the plot options config to use for the chart"""

        if 'plotOptions' not in config:
            config['plotOptions'] = {}

        if self.data_labels is not None:
            if 'series' not in config['plotOptions']:
                config['plotOptions']['series'] = {}

            if 'dataLabels' not in config['plotOptions']['series']:
                config['plotOptions']['series']['dataLabels'] = {}

            config['plotOptions']['series']['dataLabels']['enabled'] = self.data_labels

            if self.data_label_format:
                config['plotOptions']['series']['dataLabels']['format'] = self.data_label_format

        if self.colour_by_point is not None:
            if 'bar' not in config['plotOptions']:
                config['plotOptions']['bar'] = {}

            if 'column' not in config['plotOptions']:
                config['plotOptions']['column'] = {}

            config['plotOptions']['bar']['colorByPoint'] = self.colour_by_point
            config['plotOptions']['column']['colorByPoint'] = self.colour_by_point

        return config

    def add_axis(self):
        """Add a new Axis to this chart"""

        axis = Axis(self)
        self.axis.append(axis)
        return axis

    def gen_chart_config(self, config):
        """Generates the chart config"""

        if 'chart' not in config:
            config['chart'] = {}

        if self.height is not None:
            config['chart']['height'] = self.height

        if self.full_height is not None:
            config['fullHeight'] = self.full_height

        if len(self.axis) < 1 or len(self.axis) > 1:
            return config

        if self.zoom_type is not None:
            config['chart']['zoomType'] = self.zoom_type
        elif len(self.axis) > 0:
            chart_type = self.axis[0].default_chart_type
            config['chart']['zoomType'] = 'y' if chart_type == 'bar' else 'x'

        return config

    def gen_time_config(self, config):
        """Generates the time config to use for the chart"""

        if 'time' not in config:
            config['time'] = {}

        if self.use_utc is not None:
            config['time']['useUTC'] = self.use_utc

        if self.timezone_offset is not None:
            config['time']['timezoneOffset'] = self.timezone_offset

        return config

    def gen_highcharts_config(self, config):
        """Generates the config for use with highcharts"""

        config['id'] = self.id
        config['type'] = self.chart_type

        self.gen_chart_config(config)
        self.gen_title_config(config)
        self.gen_subtitle_config(config)
        self.gen_time_config(config)
        self.gen_legend_config(config)
        self.gen_tooltip_config(config)
        self.gen_plot_config(config)

        for axis in self.axis:
            axis.gen_config(config)

        return config

    def gen_single_table_config(self, config):
        """Generates the single table config"""

        axis = self.axis[0]
        headers = [axis.x_title, axis.y_title]
        rows = []

        index = 0
        for category in axis.get_categories():
            rows.append([
                category,
                axis.series[0].data[index]
            ])

            index += 1

        return {
            'id': self.id,
            'type': self.chart_type,
            'chart': {'type': 'column'},
            'xAxis': config['xAxis'],
            'series': config['series'],
            'headers': headers,
            'rows': rows,
            'title': self.get_title(),
            'subtitle': self.get_subtitle()
        }

    def gen_multi_table_config(self, config):
        """Generates the multi table config"""

        axis = self.axis[0]
        headers = [axis.x_title]
        headers.extend([
            series.get_name()
            for series in axis.series
        ])
        headers.append('Total Stories')

        rows = [
            [category]
            for category in axis.get_categories()
        ]

        for series in axis.series:
            index = 0

            for count in series.get_data():
                rows[index].append(count)

                index += 1

        for row in rows:
            row.append(
                sum(int(count) for count in row[1:])
            )

        return {
            'id': self.id,
            'type': self.chart_type,
            'chart': {'type': 'column'},
            'xAxis': config['xAxis'],
            'series': config['series'],
            'headers': headers,
            'rows': rows,
            'title': self.get_title(),
            'subtitle': self.get_subtitle()
        }

    def gen_table_config(self, config):
        """Generates the table config"""

        self.gen_highcharts_config(config)

        if len(self.axis) == 1:
            if len(self.axis[0].series) == 1:
                self.config = self.gen_single_table_config(config)
            else:
                self.config = self.gen_multi_table_config(config)

    def gen_config(self):
        """Generates the config for this chart"""

        self.config = deepcopy(self.default_config)

        if self.chart_type == 'table':
            self.gen_table_config(self.config)
        else:
            self.gen_highcharts_config(self.config)

        return self.config

    def set_translation(self, field, title, names=None):
        """Saves the provided field translations

        :param str field: The name of the field for this translation
        :param str title: The title of the field name
        :param dict names: Map of id/qcode to display names
        """

        self.translations[field.replace('.', '_')] = {
            'title': title,
            'names': names or {}
        }

    def get_translation(self, field):
        """Helper function to get the translations for a field

        :param str field: Name of the field to get translations for
        """

        return self.translations[(field or '').replace('.', '_')] or {}

    def get_translation_title(self, field):
        """Helper function to get the translated title for a field

        :param str field: Name of the field to get translated title for
        """

        return self.get_translation(field).get('title') or field

    def get_translation_names(self, field):
        """Helper function to get the translated title for a field

        :param str field: Name of the field to get translated title for
        """

        return self.get_translation(field).get('names') or {}



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from analytics.stats.common import STAT_TYPE, OPERATION, ENTER_DESK_OPERATIONS, EXIT_DESK_OPERATIONS

from copy import deepcopy


def init(stats):
    # Clear the desk stats as we'll recalculate them here
    stats[STAT_TYPE.DESK_TRANSITIONS] = []


def store_update_fields(entry, new_update):
    pass


def process(entry, new_timeline, updates, update, stats):
    updates.setdefault('_current_task', None)

    task = entry.get('task') or {}
    operation = entry.get('operation')
    operation_created = entry.get('operation_created')

    if operation in ENTER_DESK_OPERATIONS:
        updates['_current_task'] = deepcopy(task)

        if operation == OPERATION.CREATE and updates.get('rewrite_of'):
            updates['_current_task']['entered_operation'] = OPERATION.REWRITE
        else:
            updates['_current_task']['entered_operation'] = operation

        updates['_current_task']['entered'] = operation_created
    elif operation in EXIT_DESK_OPERATIONS and updates['_current_task'] is not None:
        if not updates['_current_task'].get('user') and task.get('user'):
            updates['_current_task']['user'] = task['user']

        updates['_current_task']['exited'] = operation_created
        updates['_current_task']['exited_operation'] = operation
        updates['_current_task']['duration'] = (
            updates['_current_task']['exited'] - updates['_current_task']['entered']
        ).total_seconds()

        if updates['_current_task'].get('entered_operation') == OPERATION.MOVE and \
                updates['_current_task'].get('exited_operation') == OPERATION.PUBLISH and \
                updates['_current_task']['duration'] < 3:
            pass
        else:
            stats[STAT_TYPE.DESK_TRANSITIONS].append(updates['_current_task'])

        updates['_current_task'] = None


def complete(stats, updates):
    num_desk_transitions = len(stats.get(STAT_TYPE.DESK_TRANSITIONS) or [])
    if num_desk_transitions < 1:
        stats[STAT_TYPE.DESK_TRANSITIONS] = None
        updates['num_desk_transitions'] = 0
    else:
        updates['num_desk_transitions'] = num_desk_transitions



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
#  Copyright 2013-2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from superdesk.celery_app import celery
from superdesk.default_settings import env
from superdesk.default_settings import crontab

from .archive_statistics import ArchiveStatisticsResource, ArchiveStatisticsService
from .gen_archive_statistics import GenArchiveStatistics
from .featuremedia_updates import *  # noqa


def init_app(app):
    if not app.config.get('STATISTICS_MONGO_DBNAME'):
        app.config['STATISTICS_MONGO_DBNAME'] = env('STATISTICS_MONGO_DBNAME', 'statistics')

    db_name = app.config['STATISTICS_MONGO_DBNAME']

    if not app.config.get('STATISTICS_MONGO_URI'):
        app.config['STATISTICS_MONGO_URI'] = env(
            'STATISTICS_MONGO_URI',
            'mongodb://localhost/%s' % db_name
        )

    if not app.config.get('STATISTICS_ELASTIC_URL'):
        app.config['STATISTICS_ELASTIC_URL'] = env(
            'STATISTICS_ELASTIC_URL',
            app.config['ELASTICSEARCH_URL']
        )

    if not app.config.get('STATISTICS_ELASTIC_INDEX'):
        app.config['STATISTICS_ELASTIC_INDEX'] = env('STATISTICS_ELASTIC_INDEX', db_name)

    init_gen_stats_task(app)

    endpoint_name = ArchiveStatisticsResource.endpoint_name
    service = ArchiveStatisticsService(
        endpoint_name,
        backend=superdesk.get_backend()
    )
    ArchiveStatisticsResource(
        endpoint_name,
        app=app,
        service=service
    )


def init_gen_stats_task(app):
    # Check the application config to see if archive stats is enabled
    if not app.config.get('ANALYTICS_ENABLE_ARCHIVE_STATS', False):
        return

    # Make sure the BEAT_SCHEDULE are set
    if not app.config.get('CELERY_BEAT_SCHEDULE'):
        app.config['CELERY_BEAT_SCHEDULE'] = {}

    # If the celery schedule is not configured, then set the default now
    if not app.config['CELERY_BEAT_SCHEDULE'].get('analytics:gen_archive_stats'):
        app.config['CELERY_BEAT_SCHEDULE']['analytics:gen_archive_stats'] = {
            'task': 'analytics.stats.gen_archive_stats',
            'schedule': crontab(minute='0')  # Runs every hour
        }


@celery.task(soft_time_limit=600)
def gen_archive_stats():
    GenArchiveStatistics().run()



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk import get_resource_service
from superdesk.utc import local_to_utc

from analytics.tests import TestCase
from analytics.commands.send_scheduled_reports import SendScheduledReports
from analytics.common import MIME_TYPES
from analytics.email_report.email_report import EmailReportService

from datetime import datetime
from flask import current_app as app
from dateutil.rrule import rrule, HOURLY
import pytz
from unittest import mock
from os import urandom
from base64 import b64encode, b64decode


def to_naive(date_str):
    return datetime.strptime(date_str, '%Y-%m-%dT%H')


def to_utc(date_str):
    return local_to_utc(
        app.config['DEFAULT_TIMEZONE'],
        datetime.strptime(date_str, '%Y-%m-%dT%H')
    )


def to_local(date_str):
    local_tz = pytz.timezone(app.config['DEFAULT_TIMEZONE'])
    local_datetime = datetime.strptime(date_str, '%Y-%m-%dT%H')

    return local_tz.localize(local_datetime)


mock_file = b64encode(bytearray(urandom(123)))
mock_csv = b64encode('Source,Domestic Sport,Finance\r\nAAP,2,3\r\nAP,5,10\r\n'.encode('UTF-8'))
mock_array = [{
    'file': b64encode(bytearray(urandom(456))),
    'mimetype': MIME_TYPES.PNG,
    'filename': 'chart_1.png'
}, {
    'file': b64encode(bytearray(urandom(789))),
    'mimetype': MIME_TYPES.PNG,
    'filename': 'chart_2.png'
}]
mock_users = [{'_id': 'user1'}]
mock_vocabs = [{
    '_id': 'categories',
    'items': [
        {'is_active': True, 'name': 'Domestic Sport', 'qcode': 's'},
        {'is_active': True, 'name': 'Finance', 'qcode': 'f'}
    ]
}]
mock_saved_reports = [{
    '_id': 'srep1',
    'name': 'Saved Report',
    'report': 'content_publishing_report',
    'params': {
        'dates': {
            'filter': 'range',
            'start': '2018-06-01',
            'end': '2018-06-30'
        },
        'aggs': {
            'group': {
                'field': 'source',
                'size': 0
            },
            'subgroup': {
                'field': 'anpa_category.qcode',
                'size': 0
            }
        }
    },
    'user': 'user1'
}]


class SendScheduleReportTestCase(TestCase):
    def setUp(self):
        super().setUp()
        self.app.config['DEFAULT_TIMEZONE'] = 'Australia/Sydney'
        self.app.config['ADMINS'] = ['superdesk@test.com']
        self.should_send = SendScheduledReports().should_send_report

    def _test(self, report, start, end, expected_hits):
        count = 0
        for now in rrule(HOURLY, dtstart=to_naive(start), until=to_naive(end)):
            local_tz = pytz.timezone(app.config['DEFAULT_TIMEZONE'])
            now_local = local_tz.localize(now)

            response = self.should_send(report, now_local)

            self.assertEqual(
                response,
                now_local in expected_hits,
                '{} not in {}'.format(now_local, [str(d) for d in expected_hits])
            )

            if response:
                # Update the last sent time to now
                report['_last_sent'] = local_to_utc(app.config['DEFAULT_TIMEZONE'], now_local)
                count += 1

        self.assertEqual(len(expected_hits), count)

    @mock.patch(
        'analytics.email_report.email_report.generate_report',
        return_value=mock_file
    )
    def test_run_hourly_png(self, mocked):
        with self.app.app_context():
            self.app.data.insert('users', mock_users)
            self.app.data.insert('vocabularies', mock_vocabs)
            self.app.data.insert('saved_reports', mock_saved_reports)
            self.app.data.insert('scheduled_reports', [{
                '_id': 'sched1',
                'name': 'Scheduled Report',
                'saved_report': 'srep1',
                'schedule': {'frequency': 'hourly'},
                'transmitter': 'email',
                'mimetype': MIME_TYPES.PNG,
                'extra': {'body': 'This is a test email'},
                'recipients': ['superdesk@localhost.com'],
                'report_width': 1200,
                'active': True
            }])

            scheduled_service = get_resource_service('scheduled_reports')

            report = scheduled_service.find_one(req=None, _id='sched1')
            self.assertNotIn('_last_sent', report)

            # Simulate running every hour for a few hours
            start_date = to_naive('2018-06-30T00')
            end_date = to_naive('2018-06-30T03')
            local_tz = pytz.timezone(app.config['DEFAULT_TIMEZONE'])
            with self.app.mail.record_messages() as outbox:
                for now in rrule(HOURLY, dtstart=start_date, until=end_date):
                    now_local = local_tz.localize(now)
                    now_utc = local_to_utc(app.config['DEFAULT_TIMEZONE'], now_local)

                    SendScheduledReports().run(now_utc)

                    # _last sent is updated
                    report = scheduled_service.find_one(req=None, _id='sched1')
                    self.assertEqual(report.get('_last_sent'), now_utc)

                # Test that the command sent emails across the 4 iterations
                self.assertEqual(len(outbox), 4)

                # Test the first email
                self.assertEqual(outbox[0].sender, 'superdesk@test.com')
                self.assertEqual(outbox[0].subject, 'Superdesk Analytics - Scheduled Report')
                self.assertTrue(outbox[0].body.startswith('\nThis is a test email'))
                self.assertTrue('This is a test email' in outbox[0].html)
                self.assertTrue('<img src="cid:' in outbox[0].html)

                self.assertEqual(outbox[0].recipients, ['superdesk@localhost.com'])

                # Test attachment
                self.assertEqual(len(outbox[0].attachments), 1)
                self.assertEqual(outbox[0].attachments[0].data, b64decode(mock_file))
                self.assertEqual(
                    outbox[0].attachments[0].content_type,
                    '{}; name="chart_1.png"'.format(MIME_TYPES.PNG)
                )
                self.assertEqual(outbox[0].attachments[0].filename, 'chart_1.png')

            report = scheduled_service.find_one(req=None, _id='sched1')
            self.assertEqual(report.get('_last_sent'), to_utc('2018-06-30T03'))

    @mock.patch(
        'analytics.email_report.email_report.generate_report',
        return_value=mock_file
    )
    def test_run_daily_jpeg(self, mocked):
        with self.app.app_context():
            self.app.data.insert('users', mock_users)
            self.app.data.insert('vocabularies', mock_vocabs)
            self.app.data.insert('saved_reports', mock_saved_reports)
            self.app.data.insert('scheduled_reports', [{
                '_id': 'sched1',
                'name': 'Scheduled Report',
                'saved_report': 'srep1',
                'schedule': {
                    'frequency': 'daily',
                    'hour': 1
                },
                'transmitter': 'email',
                'mimetype': MIME_TYPES.JPEG,
                'extra': {'body': 'This is a test email'},
                'recipients': ['superdesk@localhost.com'],
                'report_width': 1200,
                'active': True
            }])

            scheduled_service = get_resource_service('scheduled_reports')

            report = scheduled_service.find_one(req=None, _id='sched1')
            self.assertNotIn('_last_sent', report)

            # Simulate running every hour for a few hours
            start_date = to_naive('2018-06-30T00')
            end_date = to_naive('2018-06-30T03')
            should_have_updated = False
            with self.app.mail.record_messages() as outbox:
                for now in rrule(HOURLY, dtstart=start_date, until=end_date):
                    local_tz = pytz.timezone(app.config['DEFAULT_TIMEZONE'])
                    now_local = local_tz.localize(now)
                    now_utc = local_to_utc(app.config['DEFAULT_TIMEZONE'], now_local)

                    SendScheduledReports().run(now_utc)

                    # _last sent is updated
                    report = scheduled_service.find_one(req=None, _id='sched1')

                    if not should_have_updated:
                        should_have_updated = True
                        self.assertNotIn('_last_sent', report)
                    else:
                        self.assertEqual(report.get('_last_sent'), to_utc('2018-06-30T01'))

                # Test that the command sent only 1 email
                self.assertEqual(len(outbox), 1)

                # Test attachment
                self.assertEqual(len(outbox[0].attachments), 1)
                self.assertEqual(
                    outbox[0].attachments[0].content_type,
                    '{}; name="chart_1.jpeg"'.format(MIME_TYPES.JPEG)
                )
                self.assertEqual(outbox[0].attachments[0].filename, 'chart_1.jpeg')

            report = scheduled_service.find_one(req=None, _id='sched1')
            self.assertEqual(report.get('_last_sent'), to_utc('2018-06-30T01'))

    @mock.patch(
        'analytics.email_report.email_report.generate_report',
        return_value=mock_csv
    )
    def test_email_csv(self, mocked):
        with self.app.app_context():
            self.app.data.insert('users', mock_users)
            self.app.data.insert('vocabularies', mock_vocabs)
            self.app.data.insert('saved_reports', mock_saved_reports)
            self.app.data.insert('scheduled_reports', [{
                '_id': 'sched1',
                'name': 'Scheduled Report',
                'saved_report': 'srep1',
                'schedule': {'frequency': 'hourly'},
                'transmitter': 'email',
                'mimetype': MIME_TYPES.CSV,
                'extra': {'body': 'This is a test email'},
                'recipients': ['superdesk@localhost.com'],
                'active': True
            }])

            with self.app.mail.record_messages() as outbox:
                SendScheduledReports().run('2018-06-30T00')

                self.assertEqual(len(outbox), 1)

                # Test attachment
                self.assertEqual(len(outbox[0].attachments), 1)
                self.assertEqual(
                    outbox[0].attachments[0].content_type,
                    '{}; name="chart_1.csv"'.format(MIME_TYPES.CSV)
                )
                self.assertEqual(outbox[0].attachments[0].filename, 'chart_1.csv')
                self.assertEqual(outbox[0].attachments[0].data, b64decode(mock_csv))

    @mock.patch.object(
        EmailReportService,
        '_gen_attachments',
        return_value=mock_array
    )
    def test_email_multiple_attachments(self, mock):
        with self.app.app_context():
            self.app.data.insert('users', mock_users)
            self.app.data.insert('vocabularies', mock_vocabs)
            self.app.data.insert('saved_reports', [{
                '_id': 'srep1',
                'name': 'Saved Report',
                'report': 'content_publishing_report',
                'params': {
                    'dates': {
                        'filter': 'range',
                        'start': '2018-06-01',
                        'end': '2018-06-30',
                    },
                    'chart_type': 'pie'
                },
                'user': 'user1'
            }])
            self.app.data.insert('scheduled_reports', [{
                '_id': 'sched1',
                'name': 'Scheduled Report',
                'saved_report': 'srep1',
                'schedule': {'frequency': 'hourly'},
                'transmitter': 'email',
                'mimetype': MIME_TYPES.PNG,
                'extra': {'body': 'This is a test email'},
                'recipients': ['superdesk@localhost.com'],
                'active': True
            }])

            with self.app.mail.record_messages() as outbox:
                SendScheduledReports().run('2018-06-30T00')

                self.assertEqual(len(outbox), 1)

                # Test attachment
                self.assertEqual(len(outbox[0].attachments), 2)
                self.assertEqual(
                    outbox[0].attachments[0].content_type,
                    '{}; name="chart_1.png"'.format(MIME_TYPES.PNG)
                )
                self.assertEqual(outbox[0].attachments[0].filename, 'chart_1.png')
                self.assertEqual(outbox[0].attachments[0].data, b64decode(mock_array[0].get('file')))

                self.assertEqual(
                    outbox[0].attachments[1].content_type,
                    '{}; name="chart_2.png"'.format(MIME_TYPES.PNG)
                )
                self.assertEqual(outbox[0].attachments[1].filename, 'chart_2.png')
                self.assertEqual(outbox[0].attachments[1].data, b64decode(mock_array[1].get('file')))

    def test_send_report_hourly(self):
        # Test every hour
        self._test(
            report={'schedule': {'frequency': 'hourly', 'hour': -1}},
            start='2018-06-30T00',
            end='2018-06-30T23',
            expected_hits=[to_local('2018-06-30T{}'.format(hour)) for hour in range(0, 24)]
        )

        # Test every hour, already sent this hour
        self._test(
            report={
                'schedule': {'frequency': 'hourly', 'hour': -1},
                '_last_sent': to_utc('2018-06-30T13')
            },
            start='2018-06-30T00',
            end='2018-06-30T23',
            expected_hits=[to_local('2018-06-30T{}'.format(hour)) for hour in range(14, 24)]
        )

    def test_send_report_hour_of_the_day(self):
        # Test exact hour
        self._test(
            report={'schedule': {'frequency': 'daily', 'hour': 10}},
            start='2018-06-30T00',
            end='2018-06-30T23',
            expected_hits=[to_local('2018-06-30T10')]
        )

        # Test exact hour, not sent this hour
        report = {
            'schedule': {'frequency': 'daily', 'hour': 10},
            '_last_sent': to_local('2018-06-30T09')
        }
        self._test(
            report=report,
            start='2018-06-30T00',
            end='2018-06-30T23',
            expected_hits=[to_local('2018-06-30T10')]
        )
        # Test running this day again
        self._test(
            report=report,
            start='2018-06-30T00',
            end='2018-06-30T23',
            expected_hits=[]
        )

        # Test exact hour, already sent this hour
        self._test(
            report={
                'schedule': {'frequency': 'daily', 'hour': 10},
                '_last_sent': to_local('2018-06-30T10')
            },
            start='2018-06-30T00',
            end='2018-06-30T23',
            expected_hits=[]
        )

    def test_send_report_daily(self):
        # Every day at 8am
        self._test(
            report={'schedule': {'frequency': 'daily', 'hour': 8}},
            start='2018-06-01T00',
            end='2018-06-30T23',
            expected_hits=[
                to_local('2018-06-{}T08'.format(day)) for day in range(1, 31)
            ]
        )

    def test_send_report_weekly(self):
        # Every Monday and Wednesday @ 4pm
        self._test(
            report={
                'schedule': {
                    'frequency': 'weekly',
                    'hour': 16,
                    'week_days': ['Monday', 'Wednesday']
                }
            },
            start='2018-06-01T00',
            end='2018-06-30T23',
            expected_hits=[
                to_local('2018-06-04T16'),  # Monday
                to_local('2018-06-06T16'),  # Wednesday
                to_local('2018-06-11T16'),  # Monday
                to_local('2018-06-13T16'),  # Wednesday
                to_local('2018-06-18T16'),  # Monday
                to_local('2018-06-20T16'),  # Wednesday
                to_local('2018-06-25T16'),  # Monday
                to_local('2018-06-27T16'),  # Wednesday
            ]
        )

    def test_send_report_week_days(self):
        # Every Monday->Frday @ 2pm
        self._test(
            report={'schedule': {
                'frequency': 'weekly',
                'hour': 14,
                'week_days': ['Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday']
            }},
            start='2018-06-01T00',
            end='2018-06-30T23',
            expected_hits=[
                to_local('2018-06-01T14'),
                to_local('2018-06-04T14'),
                to_local('2018-06-05T14'),
                to_local('2018-06-06T14'),
                to_local('2018-06-07T14'),
                to_local('2018-06-08T14'),
                to_local('2018-06-11T14'),
                to_local('2018-06-12T14'),
                to_local('2018-06-13T14'),
                to_local('2018-06-14T14'),
                to_local('2018-06-15T14'),
                to_local('2018-06-18T14'),
                to_local('2018-06-19T14'),
                to_local('2018-06-20T14'),
                to_local('2018-06-21T14'),
                to_local('2018-06-22T14'),
                to_local('2018-06-25T14'),
                to_local('2018-06-26T14'),
                to_local('2018-06-27T14'),
                to_local('2018-06-28T14'),
                to_local('2018-06-29T14'),
            ]
        )

    def test_send_report_week_ends(self):
        # Every Saturday and Sunday @ 9am
        self._test(
            report={
                'schedule': {
                    'frequency': 'weekly',
                    'hour': 9,
                    'week_days': ['Saturday', 'Sunday']
                }
            },
            start='2018-06-01T00',
            end='2018-07-01T00',
            expected_hits=[
                to_local('2018-06-02T09'),
                to_local('2018-06-03T09'),
                to_local('2018-06-09T09'),
                to_local('2018-06-10T09'),
                to_local('2018-06-16T09'),
                to_local('2018-06-17T09'),
                to_local('2018-06-23T09'),
                to_local('2018-06-24T09'),
                to_local('2018-06-30T09')
            ]
        )

    def test_send_report_monthly(self):
        self._test(
            report={
                'schedule': {
                    'frequency': 'monthly',
                    'hour': 0,
                    'day': 1
                }
            },
            start='2018-01-01T00',
            end='2018-12-31T23',
            expected_hits=[
                to_local('2018-01-01T00'),
                to_local('2018-02-01T00'),
                to_local('2018-03-01T00'),
                to_local('2018-04-01T00'),
                to_local('2018-05-01T00'),
                to_local('2018-06-01T00'),
                to_local('2018-07-01T00'),
                to_local('2018-08-01T00'),
                to_local('2018-09-01T00'),
                to_local('2018-10-01T00'),
                to_local('2018-11-01T00'),
                to_local('2018-12-01T00'),
            ]
        )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013-2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource
from superdesk.errors import SuperdeskApiError

from analytics.base_report import BaseReportService
from analytics.stats.common import ENTER_DESK_OPERATIONS, EXIT_DESK_OPERATIONS
from analytics.chart_config import SDChart, ChartConfig
from analytics.common import get_utc_offset_in_minutes, REPORT_CONFIG, CHART_TYPES

from flask import current_app as app
from datetime import datetime


class DeskActivityReportResource(Resource):
    """Desk Activity Report schema"""

    item_methods = ['GET']
    resource_methods = ['GET']
    privileges = {'GET': 'desk_activity_report'}


class DeskActivityReportService(BaseReportService):
    aggregations = {
        'operations': {
            'terms': {
                'field': 'stats.timeline.operation',
                'size': 0
            }
        }
    }
    repos = ['archive_statistics']
    histogram_source_field = 'stats.timeline.operation_created'
    date_filter_field = 'versioncreated'

    defaultConfig = {
        REPORT_CONFIG.CHART_TYPES: {
            CHART_TYPES.BAR: {'enabled': True},
            CHART_TYPES.COLUMN: {'enabled': True},
            CHART_TYPES.LINE: {'enabled': True},
            CHART_TYPES.AREA: {'enabled': True},
            CHART_TYPES.SCATTER: {'enabled': True},
            CHART_TYPES.SPLINE: {'enabled': True},

            # Disable Table as generating tables for time base charts is currently not supported
            # The 'report_configs' endpoint will omit it from the result
            # CHART_TYPES.TABLE: {'enabled': False}
        }
    }

    def get_request_aggregations(self, params, args):
        aggs = super().get_request_aggregations(params, args)
        params = args.get('params') or {}
        lt, gte, time_zone = self._es_get_date_filters(params)

        desk_id = (args.get('params') or {}).get('desk')

        if not desk_id:
            raise SuperdeskApiError.badRequestError('Desk must be provided')

        new_aggs = {
            'timeline': {
                'nested': {'path': 'stats.timeline'},
                'aggs': {
                    'desk_filter': {
                        'filter': {
                            'term': {'stats.timeline.task.desk': desk_id},
                        },
                        'aggs': {
                            'timeline_filter': {
                                'filter': {
                                    'range': {
                                        'stats.timeline.operation_created': {
                                            'gte': gte,
                                            'lt': lt,
                                            'time_zone': time_zone
                                        }
                                    }
                                },
                                'aggs': aggs
                            }
                        }

                    }
                }
            }
        }

        return new_aggs

    def _get_filters(self, repos, invisible_stages):
        return None

    def get_elastic_index(self, types):
        return 'statistics'

    def generate_report(self, docs, args):
        aggregations = getattr(docs, 'hits', {}).get('aggregations') or {}
        desk_filter = ((aggregations.get('timeline') or {}).get('desk_filter') or {})
        agg_dates = (desk_filter.get('timeline_filter') or {}).get('dates') or {}
        date_buckets = agg_dates.get('buckets') or []

        if len(date_buckets) < 1:
            return {}

        report = {
            'start': date_buckets[0].get('key_as_string'),
            'start_epoch': date_buckets[0].get('key'),
            'interval': self.get_histogram_interval_ms(args),
            'incoming': [],
            'outgoing': [],
            'histogram': []
        }

        for bucket in date_buckets:
            op_buckets = (bucket.get('operations') or {}).get('buckets') or []
            histogram = {
                'interval': bucket.get('key_as_string'),
                'epoch': bucket.get('key'),
                'incoming': {
                    key: 0
                    for key in ENTER_DESK_OPERATIONS
                },
                'outgoing': {
                    key: 0
                    for key in EXIT_DESK_OPERATIONS
                },
                'other': {}
            }

            incoming = 0
            outgoing = 0

            for operation in op_buckets:
                key = operation.get('key')
                doc_count = operation.get('doc_count')

                if key in ENTER_DESK_OPERATIONS:
                    incoming += doc_count
                    histogram['incoming'][key] = doc_count
                elif key in EXIT_DESK_OPERATIONS:
                    outgoing += doc_count
                    histogram['outgoing'][key] = doc_count
                else:
                    histogram['other'][key] = doc_count

            report['incoming'].append(incoming)
            report['outgoing'].append(outgoing)
            report['histogram'].append(histogram)

        return report

    def generate_highcharts_config(self, docs, args):
        params = args.get('params') or {}
        report = self.generate_report(docs, args)
        histogram = params.get('histogram') or {}

        def gen_chart_config():
            chart_params = params.get('chart') or {}

            if chart_params.get('title'):
                title = chart_params['title']
            else:
                interval = histogram.get('interval') or 'daily'
                if interval == 'hourly':
                    title = 'Hourly Desk Activity'
                else:
                    title = 'Daily Desk Activity'

            if chart_params.get('subtitle'):
                subtitle = chart_params['subtitle']
            else:
                subtitle = ChartConfig.gen_subtitle_for_dates(params)

            # Calculate the UTC Offset in minutes for the start date of the results
            # This will cause an issue if a report crosses over the daylight savings change
            # Any data after the daylight savings change will be 1 hour out
            timezone_offset = get_utc_offset_in_minutes(
                datetime.utcfromtimestamp(int(report['start_epoch'] / 1000))
            )

            chart = SDChart.Chart(
                'desk_activity_report',
                title=title,
                subtitle=subtitle,
                chart_type='highcharts',
                start_of_week=app.config.get('START_OF_WEEK') or 0,
                timezone_offset=timezone_offset,
                use_utc=False,
                legend_title='Desk Transitions',
                default_config=ChartConfig.defaultConfig
            )

            chart.set_translation('desk_transition', 'Desk Transitions', {
                'incoming': 'Incoming',
                'outgoing': 'Outgoing'
            })

            axis = chart.add_axis().set_options(
                y_title='Desk Activity',
                type='datetime',
                default_chart_type=chart_params.get('type') or 'column',
                point_start=report.get('start_epoch'),
                point_interval=report.get('interval'),
                stack_labels=False
            )

            axis.add_series().set_options(
                field='desk_transition',
                name='incoming',
                data=report.get('incoming')
            )

            axis.add_series().set_options(
                field='desk_transition',
                name='outgoing',
                data=report.get('outgoing')
            )

            return chart.gen_config()

        def gen_table_config():
            headers = [
                'Date/Time' if histogram.get('interval') == 'hourly' else 'Date',

                'Total Incoming',
                'Create',
                'Fetch',
                'Duplicate',
                'Sent To',
                'Deschedule',
                'Unspike',

                'Total Outgoing',
                'Publish',
                'Spike',
                'Sent From',
                'Publish Scheduled',
                'Publish Embargo'
            ]

            rows = []

            for activity in report.get('histogram'):
                interval = datetime.strptime(
                    activity.get('interval'),
                    '%Y-%m-%dT%H:%M:%S'
                )

                row = [interval.strftime(
                    '%b %-d %H:%M' if histogram.get('interval') == 'hourly' else '%b %-d'
                )]

                incoming = activity.get('incoming') or {}
                incoming_rows = [
                    incoming.get(field) or 0
                    for field in ENTER_DESK_OPERATIONS
                ]
                row.append(sum(incoming_rows))
                row.extend(incoming_rows)

                outgoing = activity.get('outgoing') or {}
                outgoing_rows = [
                    outgoing.get(field) or 0
                    for field in EXIT_DESK_OPERATIONS
                ]
                row.append(sum(outgoing_rows))
                row.extend(outgoing_rows)

                rows.append(row)

            return {
                'id': 'desk_activity_report_table',
                'type': 'table',
                'chart': {'type': 'column'},
                'headers': headers,
                'title': 'Desk Activity',
                'rows': rows
            }

        report['highcharts'] = [
            gen_chart_config(),
            gen_table_config()
        ]

        return report



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013-2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from flask import current_app as app
from analytics.base_report import BaseReportService


class StatsReportService(BaseReportService):
    repos = ['archive_statistics']

    def get_elastic_index(self, types):
        return app.config.get('STATISTICS_ELASTIC_INDEX') or app.config.get('STATISTICS_MONGO_DBNAME') or 'statistics'

    def get_es_stats_type(self, query, params):
        query['must'].append({
            'term': {'stats_type': 'archive'}
        })

    def _es_base_query(self, query, params):
        super()._es_base_query(query, params)
        self.get_es_stats_type(query, params)

    def _get_filters(self, repos, invisible_stages):
        return None

    def _get_es_query_funcs(self):
        funcs = super()._get_es_query_funcs()

        funcs.update({
            'desk_transitions': {'query': self._es_filter_desk_transitions},
            'user_locks': {'query': self._es_filter_user_locks},
            'publish_pars': {'query': self._es_filter_publish_pars}
        })

        return funcs

    def _es_filter_desks(self, query, desks, must, params):
        query[must].append({
            'nested': {
                'path': 'stats.timeline',
                'query': {
                    'terms': {'stats.timeline.task.desk': desks}
                }
            }
        })

    def _es_filter_users(self, query, users, must, params):
        query[must].append({
            'nested': {
                'path': 'stats.timeline',
                'query': {
                    'terms': {'stats.timeline.task.user': users}
                }
            }
        })

    def _es_filter_stages(self, query, stages, must, params):
        query[must].append({
            'nested': {
                'path': 'stats.timeline',
                'query': {
                    'terms': {'stats.timeline.task.stage': stages}
                }
            }
        })

    def _es_filter_desk_transitions(self, query, value, must, params):
        if not isinstance(value, dict):
            return

        if value.get('min') or value.get('max'):
            ranges = {}

            if value.get('min'):
                ranges['gte'] = value['min']

            if value.get('max'):
                ranges['lte'] = value['ax']

            query[must].append({
                'range': {
                    'num_desk_transitions': ranges
                }
            })

        if value.get('enter'):
            query[must].append({
                'nested': {
                    'path': 'stats.desk_transitions',
                    'query': {
                        'terms': {'stats.desk_transitions.entered_operation': value['enter']}
                    }
                }
            })

        if value.get('exit'):
            query[must].append({
                'nested': {
                    'path': 'stats.desk_transitions',
                    'query': {
                        'terms': {'stats.desk_transitions.exited_operation': value['exit']}
                    }
                }
            })

    def _es_filter_user_locks(self, query, user, must, params):
        lt, gte, time_zone = self._es_get_date_filters(params)

        query[must].append({
            'nested': {
                'path': 'stats.timeline',
                'query': {
                    'bool': {
                        'must': [
                            {'term': {'stats.timeline.task.user': user}},
                            {'terms': {
                                'stats.timeline.operation': [
                                    'item_lock',
                                    'item_unlock'
                                ]
                            }},
                            {'range': {
                                'stats.timeline.operation_created': {
                                    'gte': gte,
                                    'lt': lt,
                                    'time_zone': time_zone
                                }
                            }}
                        ]
                    }
                }
            }
        })

    def _es_filter_publish_pars(self, query, value, must, params):
        if value:
            query[must].append({
                'nested': {
                    'path': 'stats.timeline',
                    'query': {
                        'filtered': {
                            'filter': {
                                'and': [{
                                    'term': {'stats.timeline.operation': 'publish'},
                                }, {
                                    'term': {'stats.timeline.par_count': value}
                                }]
                            }
                        }
                    }
                }
            })



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk import get_resource_service, json
from superdesk.metadata.item import ITEM_STATE, CONTENT_STATE
from superdesk.tests import TestCase

from analytics import init_app

from eve.utils import ParsedRequest
from werkzeug.datastructures import ImmutableMultiDict
import mock

aggregation_response = {
    'aggregations': {
        'source_category': {
            'buckets': [{
                'key': 'AAP',
                'doc_count': 2,
                'category': {
                    'buckets': [{
                        'key': 'Advisories',
                        'doc_count': 2
                    }]
                }
            }]
        }
    }
}


def mock_get_timezone_offset(local_tz_name, utc_datetime):
    return '+1000'


@mock.patch('analytics.base_report.get_timezone_offset', mock_get_timezone_offset)
class BaseReportServiceTestCase(TestCase):
    def setUp(self):
        self.maxDiff = None

        with self.app.app_context():
            init_app(self.app)
            self.app.config['DEFAULT_TIMEZONE'] = 'Australia/Sydney'
            self.service = get_resource_service('analytics_test_report')

            self.app.data.insert('vocabularies', [{
                '_id': 'categories',
                'items': [
                    {'is_active': True, 'name': 'Domestic Sport', 'qcode': 's'},
                    {'is_active': True, 'name': 'Finance', 'qcode': 'f'},
                    {'is_active': True, 'name': 'Advisories', 'qcode': 'v'}
                ]
            }])

            self.app.data.insert('published', [
                {
                    '_id': 'item1', 'task': {'desk': 'de1', 'stage': 'st1'}, ITEM_STATE: CONTENT_STATE.PUBLISHED,
                    'anpa_category': [{'qcode': 'v', 'name': 'Advisories'}], 'source': 'AAP'
                }
            ])

    def assert_bool_query(self, query, result):
        self.assertEqual(query, {
            'source': {
                'query': {
                    'filtered': {
                        'filter': {
                            'bool': result
                        }
                    }
                },
                'sort': [{'versioncreated': 'desc'}],
                'size': 0,
                'from': 0
            },
            'max_results': 0,
            'page': 1
        })

    def test_get_aggregation_buckets(self):
        with self.app.app_context():
            # Test source_category aggregation
            self.assertEqual(
                self.service.get_aggregation_buckets(aggregation_response, ['source_category']),
                {
                    'source_category': [{
                        'key': 'AAP',
                        'doc_count': 2,
                        'category': {
                            'buckets': [{
                                'key': 'Advisories',
                                'doc_count': 2
                            }]
                        }
                    }]
                }
            )

            # Test unknown aggregation returns an empty array
            self.assertEqual(
                self.service.get_aggregation_buckets(aggregation_response, ['test']),
                {'test': []}
            )

    def test_generate_elastic_query_default(self):
        with self.app.app_context():
            query = self.service.generate_elastic_query({})
            self.assert_bool_query(query, {
                'must': [],
                'must_not': []
            })

    def test_generate_elastic_query_from_date_object(self):
        with self.app.app_context():
            # DateFilters - Range
            query = self.service.generate_elastic_query({
                'params': {
                    'dates': {
                        'filter': 'range',
                        'start': '2018-06-01',
                        'end': '2018-06-30'
                    }
                }
            })
            self.assert_bool_query(query, {
                'must': [{
                    'range': {
                        'versioncreated': {
                            'lt': '2018-06-30T23:59:59+1000',
                            'gte': '2018-06-01T00:00:00+1000',
                            'time_zone': '+1000'
                        }
                    }
                }],
                'must_not': []
            })

            # DateFilters - Day
            query = self.service.generate_elastic_query({
                'params': {
                    'dates': {
                        'filter': 'day',
                        'date': '2018-06-30'
                    }
                }
            })
            self.assert_bool_query(query, {
                'must': [{
                    'range': {
                        'versioncreated': {
                            'lt': '2018-06-30T23:59:59+1000',
                            'gte': '2018-06-30T00:00:00+1000',
                            'time_zone': '+1000'
                        }
                    }
                }],
                'must_not': []
            })

            # DateFilters - Yesterday
            query = self.service.generate_elastic_query(
                {'params': {'dates': {'filter': 'yesterday'}}}
            )
            self.assert_bool_query(query, {
                'must': [{
                    'range': {
                        'versioncreated': {
                            'lt': 'now/d',
                            'gte': 'now-1d/d',
                            'time_zone': '+1000'
                        }
                    }
                }],
                'must_not': []
            })

            # DateFilters - Last Week
            query = self.service.generate_elastic_query(
                {'params': {'dates': {'filter': 'last_week'}}}
            )
            self.assert_bool_query(query, {
                'must': [{
                    'range': {
                        'versioncreated': {
                            'lt': 'now/w',
                            'gte': 'now-1w/w',
                            'time_zone': '+1000'
                        }
                    }
                }],
                'must_not': []
            })

            # DateFilters - Last Month
            query = self.service.generate_elastic_query(
                {'params': {'dates': {'filter': 'last_month'}}}
            )
            self.assert_bool_query(query, {
                'must': [{
                    'range': {
                        'versioncreated': {
                            'lt': 'now/M',
                            'gte': 'now-1M/M',
                            'time_zone': '+1000'
                        }
                    }
                }],
                'must_not': []
            })

            # DateFilters - Relative
            query = self.service.generate_elastic_query({
                'params': {
                    'dates': {
                        'filter': 'relative_hours',
                        'relative': 12
                    }
                }
            })
            self.assert_bool_query(query, {
                'must': [{
                    'range': {
                        'versioncreated': {
                            'lt': 'now',
                            'gte': 'now-12h/h',
                            'time_zone': '+1000'
                        }
                    }
                }],
                'must_not': []
            })

            # DateFilters - Relative Days
            query = self.service.generate_elastic_query({
                'params': {
                    'dates': {
                        'filter': 'relative_days',
                        'relative': 7
                    }
                }
            })
            self.assert_bool_query(query, {
                'must': [{
                    'range': {
                        'versioncreated': {
                            'lt': 'now',
                            'gte': 'now-7d/d',
                            'time_zone': '+1000'
                        }
                    }
                }],
                'must_not': []
            })

            # DateFilters - Today
            query = self.service.generate_elastic_query(
                {'params': {'dates': {'filter': 'today'}}}
            )
            self.assert_bool_query(query, {
                'must': [{
                    'range': {
                        'versioncreated': {
                            'lt': 'now',
                            'gte': 'now/d',
                            'time_zone': '+1000'
                        }
                    }
                }],
                'must_not': []
            })

            # DateFilters - This Week
            query = self.service.generate_elastic_query(
                {'params': {'dates': {'filter': 'this_week'}}}
            )
            self.assert_bool_query(query, {
                'must': [{
                    'range': {
                        'versioncreated': {
                            'lt': 'now',
                            'gte': 'now/w',
                            'time_zone': '+1000'
                        }
                    }
                }],
                'must_not': []
            })

            # DateFilters - This Month
            query = self.service.generate_elastic_query(
                {'params': {'dates': {'filter': 'this_month'}}}
            )
            self.assert_bool_query(query, {
                'must': [{
                    'range': {
                        'versioncreated': {
                            'lt': 'now',
                            'gte': 'now/M',
                            'time_zone': '+1000'
                        }
                    }
                }],
                'must_not': []
            })

    def test_generate_elastic_query_to_exclude_states(self):
        with self.app.app_context():
            query = self.service.generate_elastic_query({
                'params': {
                    'must_not': {
                        'states': {
                            'draft': False,
                            'ingested': False,
                            'in_progress': False,
                            'published': True,
                            'killed': True,
                            'corrected': True,
                            'recalled': False
                        }
                    }
                }
            })
            self.assert_bool_query(query, {
                'must': [],
                'must_not': [{
                    'terms': {'state': ['corrected', 'killed', 'published']}
                }]
            })

    def test_generate_elastic_query_for_repos(self):
        with self.app.app_context():
            query = self.service.generate_elastic_query({
                'params': {
                    'repos': {
                        'ingest': False,
                        'archive': False,
                        'published': True,
                        'archived': True
                    }
                }
            })
            self.assertEqual(query, {
                'source': {
                    'query': {'filtered': {'filter': {'bool': {
                        'must': [],
                        'must_not': []
                    }}}},
                    'sort': [{'versioncreated': 'desc'}],
                    'size': 0,
                    'from': 0
                },
                'repo': 'archived,published',
                'max_results': 0,
                'page': 1
            })

    def test_generate_elastic_query_for_desks(self):
        with self.app.app_context():
            query = self.service.generate_elastic_query(
                {'params': {'must': {'desks': ['desk1', 'desk2']}}}
            )
            self.assert_bool_query(query, {
                'must': [
                    {'terms': {'task.desk': ['desk1', 'desk2']}}
                ],
                'must_not': []
            })

            query = self.service.generate_elastic_query(
                {'params': {'must_not': {'desks': ['desk1', 'desk2']}}}
            )
            self.assert_bool_query(query, {
                'must': [],
                'must_not': [
                    {'terms': {'task.desk': ['desk1', 'desk2']}}
                ]
            })

    def test_generate_elastic_query_for_users(self):
        with self.app.app_context():
            query = self.service.generate_elastic_query(
                {'params': {'must': {'users': ['user1', 'user2']}}}
            )
            self.assert_bool_query(query, {
                'must': [
                    {'terms': {'task.user': ['user1', 'user2']}}
                ],
                'must_not': []
            })

            query = self.service.generate_elastic_query(
                {'params': {'must_not': {'users': ['user1', 'user2']}}}
            )
            self.assert_bool_query(query, {
                'must': [],
                'must_not': [
                    {'terms': {'task.user': ['user1', 'user2']}}
                ]
            })

    def test_generate_elastic_query_for_genre(self):
        with self.app.app_context():
            query = self.service.generate_elastic_query(
                {'params': {'must': {'genre': ['a', 'f']}}}
            )
            self.assert_bool_query(query, {
                'must': [
                    {'terms': {'genre.qcode': ['a', 'f']}}
                ],
                'must_not': []
            })

            query = self.service.generate_elastic_query(
                {'params': {'must_not': {'genre': ['a', 'f']}}}
            )
            self.assert_bool_query(query, {
                'must': [],
                'must_not': [
                    {'terms': {'genre.qcode': ['a', 'f']}}
                ]
            })

    def test_generate_elastic_query_for_urgency(self):
        with self.app.app_context():
            query = self.service.generate_elastic_query(
                {'params': {'must': {'urgency': [1, 3]}}}
            )
            self.assert_bool_query(query, {
                'must': [
                    {'terms': {'urgency': [1, 3]}}
                ],
                'must_not': []
            })

            query = self.service.generate_elastic_query(
                {'params': {'must_not': {'urgency': [1, 3]}}}
            )
            self.assert_bool_query(query, {
                'must': [],
                'must_not': [
                    {'terms': {'urgency': [1, 3]}}
                ]
            })

    def test_generate_elastic_query_for_ingest_providers(self):
        with self.app.app_context():
            query = self.service.generate_elastic_query(
                {'params': {'must': {'ingest_providers': ['ing1', 'ing2']}}}
            )
            self.assert_bool_query(query, {
                'must': [
                    {'terms': {'ingest_provider': ['ing1', 'ing2']}}
                ],
                'must_not': []
            })

            query = self.service.generate_elastic_query(
                {'params': {'must_not': {'ingest_providers': ['ing1', 'ing2']}}}
            )
            self.assert_bool_query(query, {
                'must': [],
                'must_not': [
                    {'terms': {'ingest_provider': ['ing1', 'ing2']}}
                ]
            })

    def test_generate_elastic_query_for_desk_stages(self):
        with self.app.app_context():
            query = self.service.generate_elastic_query(
                {'params': {'must': {'stages': ['sta1', 'sta2']}}}
            )
            self.assert_bool_query(query, {
                'must': [
                    {'terms': {'task.stage': ['sta1', 'sta2']}}
                ],
                'must_not': []
            })

            query = self.service.generate_elastic_query(
                {'params': {'must_not': {'stages': ['sta1', 'sta2']}}}
            )
            self.assert_bool_query(query, {
                'must': [],
                'must_not': [
                    {'terms': {'task.stage': ['sta1', 'sta2']}}
                ]
            })

    def test_generate_elastic_query_for_content_types(self):
        with self.app.app_context():
            query = self.service.generate_elastic_query(
                {'params': {'must': {'content_types': ['text', 'picture']}}}
            )
            self.assert_bool_query(query, {
                'must': [
                    {'terms': {'type': ['picture', 'text']}}
                ],
                'must_not': []
            })

            query = self.service.generate_elastic_query(
                {'params': {'must_not': {'content_types': ['text', 'picture']}}}
            )
            self.assert_bool_query(query, {
                'must': [],
                'must_not': [
                    {'terms': {'type': ['picture', 'text']}}
                ]
            })

    def test_generate_elastic_query_for_states(self):
        with self.app.app_context():
            query = self.service.generate_elastic_query(
                {'params': {'must': {'states': ['draft', 'ingested']}}}
            )
            self.assert_bool_query(query, {
                'must': [
                    {'terms': {'state': ['draft', 'ingested']}}
                ],
                'must_not': []
            })

            query = self.service.generate_elastic_query(
                {'params': {'must_not': {'states': ['corrected', 'killed']}}}
            )
            self.assert_bool_query(query, {
                'must': [],
                'must_not': [
                    {'terms': {'state': ['corrected', 'killed']}}
                ]
            })

    def test_generate_elastic_query_for_categories(self):
        with self.app.app_context():
            query = self.service.generate_elastic_query({
                'params': {
                    'must': {
                        'categories': {
                            'Domestic Sport': True,
                            'Finance': False,
                            'International Sport': True
                        }
                    },
                    'category_field': 'name'
                }
            })
            self.assert_bool_query(query, {
                'must': [{
                    'terms': {
                        'anpa_category.name': [
                            'Domestic Sport',
                            'International Sport'
                        ]
                    }
                }],
                'must_not': []
            })

            query = self.service.generate_elastic_query({
                'params': {
                    'must_not': {
                        'categories': {
                            'Domestic Sport': True,
                            'Finance': False,
                            'International Sport': True
                        }
                    },
                    'category_field': 'name'
                }
            })
            self.assert_bool_query(query, {
                'must': [],
                'must_not': [{
                    'terms': {
                        'anpa_category.name': [
                            'Domestic Sport',
                            'International Sport'
                        ]
                    }
                }]
            })

    def test_generate_elastic_query_for_sources(self):
        with self.app.app_context():
            query = self.service.generate_elastic_query({
                'params': {
                    'must': {
                        'sources': {
                            'AAP': True,
                            'Reuters': True,
                            'AP': False
                        }
                    }
                }
            })
            self.assert_bool_query(query, {
                'must': [{
                    'terms': {'source': ['AAP', 'Reuters']}
                }],
                'must_not': []
            })

            query = self.service.generate_elastic_query({
                'params': {
                    'must_not': {
                        'sources': {
                            'AAP': True,
                            'Reuters': True,
                            'AP': False
                        }
                    }
                }
            })
            self.assert_bool_query(query, {
                'must': [],
                'must_not': [{
                    'terms': {'source': ['AAP', 'Reuters']}
                }]
            })

    def test_generate_elastic_query_to_exclude_rewrites(self):
        with self.app.app_context():
            query = self.service.generate_elastic_query({
                'params': {
                    'must_not': {'rewrites': True}
                }
            })
            self.assert_bool_query(query, {
                'must': [],
                'must_not': [{
                    'exists': {'field': 'rewrite_of'}
                }]
            })

            query = self.service.generate_elastic_query({
                'params': {
                    'must_not': {'rewrites': False}
                }
            })
            self.assert_bool_query(query, {
                'must': [],
                'must_not': []
            })

    def test_generate_elastic_query_size(self):
        with self.app.app_context():
            query = self.service.generate_elastic_query({
                'params': {'size': 200}
            })
            self.assertEqual(query, {
                'source': {
                    'query': {'filtered': {'filter': {'bool': {
                        'must': [],
                        'must_not': []
                    }}}},
                    'sort': [{'versioncreated': 'desc'}],
                    'size': 200,
                    'from': 0
                },
                'max_results': 200,
                'page': 1
            })

    def test_get_with_request(self):
        params = {
            'source': {
                'query': {
                    'filtered': {
                        'filter': {
                            'bool': {
                                'must': [],
                                'must_not': []
                            }
                        }
                    }
                }
            },
            'params': {
                'dates': {'filter': 'yesterday'}
            },
            'repo': 'published',
            'return_type': 'text/csv'
        }
        expected_args = {
            'source': params['source'],
            'repo': params['repo'],
            'return_type': params['return_type']
        }
        request = ParsedRequest()

        # Request object with source as a json in string format
        request.args = {
            'source': json.dumps(params['source']),
            'repo': params['repo'],
            'return_type': params['return_type'],
        }
        args = self.service._get_request_or_lookup(req=request, lookup=None)
        self.assertEqual(args, expected_args)

        # Request object with args an ImmutableMultiDict
        request.args = ImmutableMultiDict({
            'source': json.dumps(params['source']),
            'repo': params['repo'],
            'return_type': params['return_type'],
        })
        args = self.service._get_request_or_lookup(req=request, lookup=None)
        self.assertEqual(args, expected_args)

        # Request object with source as a normal dict
        request.args = {
            'source': params['source'],
            'repo': params['repo'],
            'return_type': params['return_type'],
        }
        args = self.service._get_request_or_lookup(req=request, lookup=None)
        self.assertEqual(args, expected_args)

        # return_type default
        request.args = {
            'source': params['source'],
            'repo': params['repo'],
        }
        args = self.service._get_request_or_lookup(req=request, lookup=None)
        self.assertEqual(args.get('return_type'), 'aggregations')

        # Request object with params as json in string format
        expected_args = {
            'params': params['params'],
            'repo': params['repo'],
            'return_type': params['return_type']
        }
        request.args = {
            'params': json.dumps(params['params']),
            'repo': params['repo'],
            'return_type': params['return_type'],
        }
        args = self.service._get_request_or_lookup(req=request, lookup=None)
        self.assertEqual(args, expected_args)

        # Request object with params as a normal dict
        request.args = {
            'params': params['params'],
            'repo': params['repo'],
            'return_type': params['return_type'],
        }
        args = self.service._get_request_or_lookup(req=request, lookup=None)
        self.assertEqual(args, expected_args)

    def test_get_with_lookup(self):
        lookup = {
            'source': {
                'query': {
                    'filtered': {
                        'filter': {
                            'bool': {
                                'must': [],
                                'must_not': []
                            }
                        }
                    }
                }
            },
            'params': {
                'dates': {'filter': 'yesterday'}
            },
            'repo': 'published',
            'return_type': 'text/csv'
        }
        expected_args = {
            'source': lookup['source'],
            'repo': lookup['repo'],
            'return_type': lookup['return_type']
        }

        # Lookup with source as a json in string format
        args = self.service._get_request_or_lookup(
            req=None,
            source=json.dumps(lookup['source']),
            repo=lookup['repo'],
            return_type=lookup['return_type']
        )
        self.assertEqual(args, expected_args)

        # Lookup with source as a normal dict
        args = self.service._get_request_or_lookup(
            req=None,
            source=lookup['source'],
            repo=lookup['repo'],
            return_type=lookup['return_type']
        )
        self.assertEqual(args, expected_args)

        # return_type default
        args = self.service._get_request_or_lookup(
            req=None,
            source=lookup['source'],
            repo=lookup['repo']
        )
        self.assertEqual(args.get('return_type'), 'aggregations')

        # Lookup with params as json in string format
        expected_args = {
            'params': lookup['params'],
            'repo': lookup['repo'],
            'return_type': lookup['return_type']
        }
        args = self.service._get_request_or_lookup(
            req=None,
            params=json.dumps(lookup['params']),
            repo=lookup['repo'],
            return_type=lookup['return_type']
        )
        self.assertEqual(args, expected_args)

        # Lookup with params as normal dict
        args = self.service._get_request_or_lookup(
            req=None,
            params=lookup['params'],
            repo=lookup['repo'],
            return_type=lookup['return_type']
        )
        self.assertEqual(args, expected_args)

    def test_aggs_with_request(self):
        params = {
            'params': {'dates': {'filter': 'yesterday'}},
            'aggs': {
                'group': {'field': 'anpa_category.qcode'},
                'subgroup': {'field': 'urgency'}
            },
            'repo': 'published',
            'return_type': 'text/csv'
        }
        expected_args = {
            'params': params['params'],
            'aggs': params['aggs'],
            'repo': params['repo'],
            'return_type': params['return_type']
        }

        request = ParsedRequest()

        # Request object with source as a json in string format
        request.args = {
            'params': json.dumps(params['params']),
            'repo': params['repo'],
            'return_type': params['return_type'],
            'aggs': json.dumps(params['aggs'])
        }
        args = self.service._get_request_or_lookup(req=request, lookup=None)
        self.assertEqual(args, expected_args)

        # Request object with args an ImmutableMultiDict
        request.args = ImmutableMultiDict({
            'params': params['params'],
            'repo': params['repo'],
            'return_type': params['return_type'],
            'aggs': params['aggs']
        })
        args = self.service._get_request_or_lookup(req=request, lookup=None)
        self.assertEqual(args, expected_args)

        # Request object with source as a normal dict
        request.args = {
            'params': params['params'],
            'repo': params['repo'],
            'return_type': params['return_type'],
            'aggs': params['aggs']
        }
        args = self.service._get_request_or_lookup(req=request, lookup=None)
        self.assertEqual(args, expected_args)

    def test_aggs_with_lookup(self):
        lookup = {
            'params': {'dates': {'filter': 'yesterday'}},
            'aggs': {
                'group': {'field': 'anpa_category.qcode'},
                'subgroup': {'field': 'urgency'}
            },
            'repo': 'published',
            'return_type': 'text/csv'
        }
        expected_args = {
            'params': lookup['params'],
            'aggs': lookup['aggs'],
            'repo': lookup['repo'],
            'return_type': lookup['return_type']
        }

        # Lookup with aggs as a json in string format
        args = self.service._get_request_or_lookup(
            req=None,
            params=json.dumps(lookup['params']),
            repo=lookup['repo'],
            return_type=lookup['return_type'],
            aggs=json.dumps(lookup['aggs'])
        )
        self.assertEqual(args, expected_args)

        # Lookup with aggs as a normal dict
        args = self.service._get_request_or_lookup(
            req=None,
            params=lookup['params'],
            repo=lookup['repo'],
            return_type=lookup['return_type'],
            aggs=lookup['aggs']
        )
        self.assertEqual(args, expected_args)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from .planning_usage_report import PlanningUsageReportResource, PlanningUsageReportService
from analytics.common import register_report


def init_app(app):
    # Don't register this endpoint if the Planning module is not configured
    if 'planning' not in app.settings['INSTALLED_APPS']:
        return

    endpoint_name = 'planning_usage_report'
    service = PlanningUsageReportService(endpoint_name, backend=superdesk.get_backend())
    PlanningUsageReportResource(endpoint_name, app=app, service=service)
    register_report('planning_usage_report', 'planning_usage_report')

    superdesk.privilege(
        name='planning_usage_report',
        label='Analytics - Planning Usage Reports',
        description='User can view Planning Usage Reports'
    )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013-2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource
from superdesk.utc import utc_to_local

from analytics.stats.stats_report_service import StatsReportService
from analytics.chart_config import ChartConfig
from analytics.common import REPORT_CONFIG, CHART_TYPES

from flask import current_app as app
from eve_elastic.elastic import parse_date


class FeaturemdiaUpdatesReportResource(Resource):
    """Featuremedia Updates Report schema"""

    item_methods = ['GET']
    resource_methods = ['GET']
    privileges = {'GET': 'featuremedia_updates_report'}


class FeaturemediaUpdatesTimeReportService(StatsReportService):
    aggregations = None
    date_filter_field = 'versioncreated'

    defaultConfig = {
        REPORT_CONFIG.CHART_TYPES: {
            # Table is the only supported chart type for this report
            # The 'report_configs' endpoint will omit all others
            CHART_TYPES.TABLE: {'enabled': True}
        },
        REPORT_CONFIG.DEFAULT_PARAMS: {
            'chart': {'type': CHART_TYPES.TABLE}
        }
    }

    def get_request_aggregations(self, params, args):
        """Disable generating aggregations"""
        return None

    def get_es_stats_type(self, query, params):
        # Check the rewrites filter is include|exclude|only
        # then modify this query to reflect the filter
        if (params.get('rewrites') or 'include') == 'include':
            query['should'] += [{
                'bool': {
                    'must': {'term': {'stats_type': 'archive'}},
                    'must_not': [
                        {'exists': {'field': 'rewritten_by'}},
                        {'exists': {'field': 'rewrite_of'}}
                    ]
                }
            }, {
                'bool': {
                    'must': [
                        {'term': {'stats_type': 'archive_family'}},
                        {'exists': {'field': 'rewritten_by'}}
                    ],
                }
            }]

            query['minimum_should_match'] = 1
        else:
            query['must'].append({'term': {'stats_type': 'archive'}})

    def _es_set_size(self, query, params):
        """Disable setting the size"""
        query['size'] = 200

    def generate_elastic_query(self, args):
        query = super().generate_elastic_query(args)

        query['source']['query']['filtered']['filter']['bool']['must'].extend([{
            'range': {'num_featuremedia_updates': {'gt': 1}}
        }, {
            'term': {'type': 'text'}
        }])

        return query

    def generate_report(self, docs, args):
        chart_params = (args.get('params') or {}).get('chart') or {}
        report = {'items': []}

        def get_featuremedia(update):
            return (update.get('associations') or {}).get('featuremedia') or {}

        for doc in sorted(
                docs,
                key=lambda d: d.get('versioncreated'),
                reverse=(chart_params.get('sort_order') or 'desc') == 'desc'
        ):
            stats = doc.get('stats') or {}
            featuremedia_updates = stats.get('featuremedia_updates') or []

            try:
                featuremedia = get_featuremedia(
                    featuremedia_updates[0].get('update') or {}
                )
                original_image = {
                    '_id': featuremedia.get('_id'),
                    'guid': featuremedia.get('guid'),
                    'headline': featuremedia.get('headline'),
                    'alt_text': featuremedia.get('alt_text'),
                    'description_text': featuremedia.get('description_text')
                }
            except IndexError:
                # Our elastic query should have filtered out empty featuremedia updates
                # Therefor this should never happen, so we can skip this item
                continue

            item_report = {
                'firstcreated': doc.get('firstcreated'),
                'firstpublished': doc.get('firstpublished'),
                'versioncreated': doc.get('versioncreated'),
                'original_creator': doc.get('original_creator'),
                'slugline': doc.get('slugline'),
                'headline': doc.get('headline'),
                'item_type': doc.get('stats_type'),
                'source': doc.get('source'),
                'state': doc.get('state'),
                'updates': [],
                'original_image': original_image,
                '_id': doc.get('_id')
            }

            for update in featuremedia_updates:
                item_report['updates'].append({
                    'operation': update.get('operation'),
                    'operation_created': update.get('operation_created'),
                    'user': (update.get('task') or {}).get('user'),
                    'update': get_featuremedia(update.get('update') or {})
                })

            report['items'].append(item_report)

        return report

    def generate_highcharts_config(self, docs, args):
        report = self.generate_report(docs, args)

        params = args.get('params') or {}
        chart_params = params.get('chart') or {}
        items = report.get('items') or []

        title = chart_params.get('title') or 'Changes to Featuremedia'
        subtitle = chart_params.get('subtitle') or ChartConfig.gen_subtitle_for_dates(params)

        translations = ChartConfig.get_translations(['task.user', 'operation'])
        user_translations = (translations.get('task_user') or {}).get('names') or {}
        operation_translations = (translations.get('operation') or {}).get('names') or {}
        rows = []

        def gen_date_str(date):
            return utc_to_local(
                app.config['DEFAULT_TIMEZONE'],
                date
            ).strftime('%d/%m/%Y %H:%M')

        for item in items:
            original_image = item.get('original_image') or {}
            updates = []

            for update in item.get('updates') or []:
                updates.append('{} - {} by {}'.format(
                    gen_date_str(parse_date(update.get('operation_created'))),
                    operation_translations.get(update.get('operation')) or update.get('operation'),
                    user_translations.get(update.get('user')) or ''
                ))

            rows.append([
                gen_date_str(item.get('versioncreated')),
                user_translations.get(item.get('original_creator')) or '',
                item.get('slugline') or '',
                original_image.get('headline') or original_image.get('alt_text') or '',
                '<ul><li>{}</li></ul>'.format('</li><li>'.join(updates))
            ])

        return {
            'highcharts': [{
                'id': 'featuremedia_updates',
                'type': 'table',
                'title': title,
                'subtitle': subtitle,
                'headers': ['Date', 'Creator', 'Slugline', 'Original Image', 'Featuremedia History'],
                'rows': rows
            }]
        }



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from flask import json, current_app as app
from eve_elastic.elastic import set_filters, ElasticCursor

from superdesk import get_resource_service, es_utils
from superdesk.utils import ListCursor
from superdesk.utc import utcnow, get_timezone_offset
from superdesk.errors import SuperdeskApiError
from superdesk.es_utils import REPOS

from apps.search import SearchService

from analytics.common import MIME_TYPES, get_elastic_version, get_weekstart_offset_hr, DATE_FILTERS, \
    relative_to_absolute_datetime


class BaseReportService(SearchService):
    exclude_stages_with_global_read_off = True
    date_filter_field = 'versioncreated'
    histogram_source_field = 'versioncreated'

    defaultConfig = {}
    repos = REPOS

    def get_stages_to_exclude(self):
        """
        Overriding from the base SearchService so we can control which stages to include.
        """
        if self.exclude_stages_with_global_read_off:
            stages = get_resource_service('stages').get_stages_by_visibility(is_visible=False)
            return [str(stage['_id']) for stage in stages]

        return []

    def on_fetched(self, doc):
        """
        Overriding this method so the base SearchService doesn't construct custom HATEOS
        """
        pass

    def generate_report(self, docs, args):
        """
        Overwrite this method to generate a report based on the aggregation data
        """
        if args.get('aggs', 1) == 0:
            return docs

        return self.get_aggregation_buckets(docs.hits)

    def generate_highcharts_config(self, docs, args):
        """
        Overwrite this method to generate the highcharts config based on the aggregation data
        """
        return {}

    def generate_csv(self, docs, args):
        """
        Overwrite this method to generate the csv data based on the aggregation data
        """
        return {}

    def generate_html(self, docs, args):
        """
        Overwrite this method to generate the html table based on the aggregation data
        """
        return {}

    def get_aggregation_buckets(self, docs, aggregation_ids=None):
        """
        Retrieves the aggregation buckets from the documents provided
        """
        if aggregation_ids is None:
            aggregation_ids = self.aggregations.keys()

        buckets = {}

        for aggregation_id in aggregation_ids:
            aggregations = docs.get('aggregations') or {}
            buckets[aggregation_id] = (aggregations.get(aggregation_id) or {}).get('buckets') or []

        return buckets

    def get_aggregations(self, params, args):
        return self.aggregations

    def _get_histogram_aggregation(self, interval, args, aggregations):
        params = args.get('params') or {}
        lt, gte, time_zone = self._es_get_date_filters(params)

        # If the interval is weekly, then offset the buckets by the
        # starting day of the week, based on app.config['START_OF_WEEK']
        offset = 0 if interval != 'week' else get_weekstart_offset_hr()

        if lt.startswith('now'):
            extended_bounds = {
                'max': relative_to_absolute_datetime(lt, '%Y-%m-%dT%H:%M:%S'),
                'min': relative_to_absolute_datetime(gte, '%Y-%m-%dT%H:%M:%S')
            }
        else:
            extended_bounds = {
                'min': gte[:-5],  # remove timezone part
                'max': lt[:-5]  # remove timezone part
            }

        # dates.extended_bounds.min & dates.extended_bounds.max should use dates.format

        aggs = {
            'dates': {
                'date_histogram': {
                    'field': self.histogram_source_field,
                    'interval': interval,
                    'time_zone': time_zone,
                    'min_doc_count': 0,
                    'offset': '{}h'.format(offset),
                    'extended_bounds': extended_bounds,
                    'format': 'yyyy-MM-dd\'T\'HH:mm:ss'
                },
                'aggs': aggregations
            }
        }

        if get_elastic_version().startswith('1.'):
            aggs['dates']['date_histogram']['pre_zone_adjust_large_interval'] = True

        return aggs

    def get_histogram_interval(self, args):
        histogram = (args.get('params') or {}).get('histogram') or {}
        return histogram.get('interval')

    def get_histogram_interval_ms(self, args):
        interval = self.get_histogram_interval(args)

        hours = 1
        if interval == 'daily':
            hours = 24
        elif interval == 'weekly':
            hours = 168

        return 3600000 * hours  # (milliseconds in an hour) * number of hours

    def get_histogram_aggregation(self, aggs, params, args):
        interval = self.get_histogram_interval(args)

        if not interval:
            return aggs

        if interval == 'hourly':
            return self._get_histogram_aggregation('hour', args, aggs)
        elif interval == 'weekly':
            return self._get_histogram_aggregation('week', args, aggs)

        return self._get_histogram_aggregation('day', args, aggs)

    def get_request_aggregations(self, params, args):
        aggs = self.get_aggregations(params, args)
        return self.get_histogram_aggregation(aggs, params, args)

    def _get_request_or_lookup(self, req, **lookup):
        # Get the args as a dictionary from either the request or lookup object
        if getattr(req, 'args', None) is not None:
            # Convert to a normal dict (werkzeug passes through an ImmutableMultiDict)
            try:
                args = req.args.to_dict()
            except AttributeError:
                args = req.args
        else:
            args = {
                'source': lookup.get('source') or None,
                'params': lookup.get('params') or None,
                'repo': lookup.get('repo') or None,
                'return_type': lookup.get('return_type') or 'aggregations',
                'aggs': lookup.get('aggs') or None,
                'translations': lookup.get('translations') or None
            }

        # Args can either have source or params, not both
        # Also ensure the attributes are a dictionary not json string
        if args.get('source'):
            if isinstance(args['source'], str):
                args['source'] = json.loads(args['source'])
            args.pop('params', None)
        elif args.get('params'):
            if isinstance(args['params'], str):
                args['params'] = json.loads(args['params'])
            args.pop('source', None)

            # args['params']['aggs'] takes precedence over args['aggs']
            if args['params'].get('aggs'):
                args['aggs'] = args['params']['aggs']

        if 'aggs' in args:
            if isinstance(args['aggs'], str):
                args['aggs'] = json.loads(args['aggs'])
            elif args['aggs'] is None:
                del args['aggs']

        if 'translations' in args:
            if isinstance(args['translations'], str):
                args['translations'] = json.loads(args['translations'])
            elif args['translations'] is None:
                del args['translations']

        args['return_type'] = args.get('return_type', 'aggregations')

        return args

    def get_elastic_index(self, types):
        return es_utils.get_index(types)

    def run_query(self, params, args):
        query = params.get('source') or {}
        if 'query' not in query:
            query['query'] = {'filtered': {}}

        aggs = self.get_request_aggregations(params, args)
        if aggs:
            query['aggs'] = aggs

        types = params.get('repo')
        if not types:
            types = self.repos.copy()
        else:
            types = types.split(',')
            # If the repos array is still empty after filtering, then return the default repos
            types = [repo for repo in types if repo in self.repos] or self.repos.copy()

        excluded_stages = self.get_stages_to_exclude()
        filters = self._get_filters(types, excluded_stages)

        # if the system has a setting value for the maximum search depth then apply the filter
        if not app.settings['MAX_SEARCH_DEPTH'] == -1:
            query['terminate_after'] = app.settings['MAX_SEARCH_DEPTH']

        if filters:
            set_filters(query, filters)

        index = self.get_elastic_index(types)

        hits = self.elastic.es.search(
            body=query,
            index=index,
            doc_type=types,
            params={}
        )
        docs = self._get_docs(hits)

        for resource in types:
            response = {app.config['ITEMS']: [doc for doc in docs if doc['_type'] == resource]}
            getattr(app, 'on_fetched_resource')(resource, response)
            getattr(app, 'on_fetched_resource_%s' % resource)(response)

        return docs

    def get(self, req, **lookup):
        args = self._get_request_or_lookup(req, **lookup)

        if args.get('source'):
            params = {
                'source': args['source'],
                'repo': args.get('repo')
            }

            if args.get('aggs'):
                params['aggs'] = args['aggs']

        elif args.get('params'):
            params = self.generate_elastic_query(args)
            if args.get('aggs'):
                params['aggs'] = args['aggs']
        else:
            raise SuperdeskApiError.badRequestError('source/query not provided')

        docs = self.run_query(params, args)

        if args['return_type'] == 'highcharts_config':
            report = self.generate_highcharts_config(docs, args)
        elif args['return_type'] == MIME_TYPES.CSV:
            report = self.generate_csv(docs, args)
        elif args['return_type'] == MIME_TYPES.HTML:
            report = self.generate_html(docs, args)
        else:
            report = self.generate_report(docs, args)

        if 'include_items' in args and int(args['include_items']):
            report['_items'] = list(docs)

        if isinstance(report, list):
            return ListCursor(report)
        elif isinstance(report, ListCursor):
            return report
        elif isinstance(report, ElasticCursor):
            return report
        return ListCursor([report])

    def get_utc_offset(self):
        return get_timezone_offset(app.config['DEFAULT_TIMEZONE'], utcnow())

    def format_date(self, date, end_of_day=False):
        time_suffix = 'T23:59:59' if end_of_day else 'T00:00:00'
        utc_offset = self.get_utc_offset()

        return date + time_suffix + utc_offset

    def _es_get_filter_values(self, filters):
        if not isinstance(filters, dict):
            return filters

        return [name for name, value in filters.items() if value]

    def _es_filter_desks(self, query, desks, must, params):
        query[must].append({
            'terms': {'task.desk': desks}
        })

    def _es_filter_users(self, query, users, must, params):
        query[must].append({
            'terms': {'task.user': users}
        })

    def _es_filter_categories(self, query, categories, must, params):
        field = params.get('category_field') or 'qcode'

        query[must].append({
            'terms': {'anpa_category.{}'.format(field): sorted(categories)}
        })

    def _es_filter_sources(self, query, sources, must, params):
        query[must].append({
            'terms': {'source': sorted(sources)}
        })

    def _es_filter_genre(self, query, genres, must, params):
        query[must].append({
            'terms': {'genre.qcode': genres}
        })

    def _es_filter_urgencies(self, query, urgencies, must, params):
        query[must].append({
            'terms': {'urgency': urgencies}
        })

    def _es_filter_ingest_providers(self, query, ingests, must, params):
        query[must].append({
            'terms': {'ingest_provider': ingests}
        })

    def _es_filter_stages(self, query, stages, must, params):
        query[must].append({
            'terms': {'task.stage': stages}
        })

    def _es_filter_states(self, query, states, must, params):
        query[must].append({
            'terms': {'state': sorted(states)}
        })

    def _es_filter_content_types(self, query, content_types, must, params):
        query[must].append({
            'terms': {'type': sorted(content_types)}
        })

    def _es_filter_rewrites(self, query, value, must, params):
        if value:
            query[must].append({
                'exists': {'field': 'rewrite_of'}
            })

    def _es_include_rewrites(self, query, params):
        rewrites = params.get('rewrites') or 'include'

        if rewrites == 'include':
            return

        must = 'must' if rewrites == 'only' else 'must_not'

        query[must].append({
            "and": [
                {"term": {"state": "published"}},
                {"exists": {"field": "rewrite_of"}}
            ]
        })

    def _es_set_repos(self, query, params):
        query['repo'] = ','.join([
            repo
            for repo, value in sorted((params.get('repos') or {}).items())
            if value and repo
        ])

    def _es_set_size(self, query, params):
        query['size'] = params.get('size') or 0

        if 'page' in params:
            query['page'] = params['page']

    def _es_set_sort(self, query, params):
        query['sort'] = params.get('sort') or [{self.date_filter_field: 'desc'}]

    def _es_get_date_filters(self, params):
        dates = params.get('dates') or {}
        date_filter = dates.get('filter')
        if not date_filter:
            return None, None, None

        start_date = dates.get('start')
        end_date = dates.get('end')
        date = dates.get('date')
        relative = dates.get('relative')

        time_zone = self.get_utc_offset()
        lt = None
        gte = None

        if date_filter == DATE_FILTERS.RANGE:
            lt = self.format_date(end_date, True)
            gte = self.format_date(start_date)
        elif date_filter == DATE_FILTERS.DAY:
            lt = self.format_date(date, True)
            gte = self.format_date(date)
        elif date_filter == DATE_FILTERS.YESTERDAY:
            lt = 'now/d'
            gte = 'now-1d/d'
        elif date_filter == DATE_FILTERS.LAST_WEEK:
            lt = 'now/w'
            gte = 'now-1w/w'
        elif date_filter == DATE_FILTERS.LAST_MONTH:
            lt = 'now/M'
            gte = 'now-1M/M'
        elif date_filter == DATE_FILTERS.RELATIVE_HOURS:
            lt = 'now'
            gte = 'now-{}h/h'.format(relative)
        elif date_filter == DATE_FILTERS.RELATIVE_DAYS:
            lt = 'now'
            gte = 'now-{}d/d'.format(relative)
        elif date_filter == DATE_FILTERS.TODAY:
            lt = 'now'
            gte = 'now/d'
        elif date_filter == DATE_FILTERS.THIS_WEEK:
            lt = 'now'
            gte = 'now/w'
        elif date_filter == DATE_FILTERS.THIS_MONTH:
            lt = 'now'
            gte = 'now/M'
        elif date_filter == DATE_FILTERS.RELATIVE_WEEKS:
            lt = 'now'
            gte = 'now-{}w/d'.format(relative)
        elif date_filter == DATE_FILTERS.RELATIVE_MONTHS:
            lt = 'now'
            gte = 'now-{}M/d'.format(relative)
        elif date_filter == DATE_FILTERS.LAST_YEAR:
            lt = 'now/y'
            gte = 'now-1y/y'
        elif date_filter == DATE_FILTERS.THIS_YEAR:
            lt = 'now'
            gte = 'now/y'

        return lt, gte, time_zone

    def _es_filter_dates(self, query, params):
        lt, gte, time_zone = self._es_get_date_filters(params)

        if lt is not None and gte is not None:
            query['must'].append({
                DATE_FILTERS.RANGE: {
                    self.date_filter_field: {
                        'lt': lt,
                        'gte': gte,
                        'time_zone': time_zone
                    }
                }
            })

    def _get_es_query_funcs(self):
        return {
            'desks': {
                'query': self._es_filter_desks,
                'values': self._es_get_filter_values
            },
            'users': {
                'query': self._es_filter_users,
                'values': self._es_get_filter_values
            },
            'categories': {
                'query': self._es_filter_categories,
                'values': self._es_get_filter_values
            },
            'sources': {
                'query': self._es_filter_sources,
                'values': self._es_get_filter_values
            },
            'genre': {
                'query': self._es_filter_genre,
                'values': self._es_get_filter_values
            },
            'urgency': {
                'query': self._es_filter_urgencies,
                'values': self._es_get_filter_values
            },
            'ingest_providers': {
                'query': self._es_filter_ingest_providers,
                'values': self._es_get_filter_values
            },
            'stages': {
                'query': self._es_filter_stages,
                'values': self._es_get_filter_values
            },
            'content_types': {
                'query': self._es_filter_content_types,
                'values': self._es_get_filter_values
            },
            'states': {
                'query': self._es_filter_states,
                'values': self._es_get_filter_values
            },
            'rewrites': {
                'query': self._es_filter_rewrites,
                'values': self._es_get_filter_values
            },
        }

    def _es_base_query(self, query, params):
        self._es_set_repos(query, params)
        self._es_set_size(query, params)
        self._es_set_sort(query, params)
        self._es_filter_dates(query, params)
        self._es_include_rewrites(query, params)

    def generate_elastic_query(self, args):
        params = args.get('params') or {}

        query_funcs = self._get_es_query_funcs()
        query = {
            'must': [],
            'must_not': [],
            'sort': [],
            'should': []
        }

        self._es_base_query(query, params)

        for must in ['must', 'must_not']:
            for field, filters in (params.get(must) or {}).items():
                if isinstance(filters, list) and len(filters) < 1:
                    continue
                elif isinstance(filters, bool) and not filters:
                    continue
                elif filters is None:
                    continue

                funcs = query_funcs.get(field)

                values = filters if not funcs.get('values') else funcs['values'](filters)
                func = funcs.get('query')

                if (isinstance(values, list) and len(values) < 1) or not func:
                    continue

                func(query, values, must, params)

        es_query = {
            'source': {
                'query': {
                    'filtered': {
                        'filter': {
                            'bool': {
                                'must': query['must'],
                                'must_not': query['must_not']
                            },
                        },
                    },
                }
            }
        }

        query_keys = query.keys()

        if len(query.get('should') or []) > 0:
            es_query['source']['query']['filtered']['filter']['bool']['should'] = \
                query['should']

        if 'minimum_should_match' in query_keys:
            es_query['source']['query']['filtered']['filter']['bool']['minimum_should_match'] = \
                query['minimum_should_match']

        if 'size' in query_keys:
            es_query['source']['size'] = query['size']

            page = query.get('page') or 1
            es_query['source']['from'] = (page - 1) * query['size']
            es_query['page'] = query.get('page') or 1
            es_query['max_results'] = query['size']

        if 'sort' in query_keys:
            es_query['source']['sort'] = query['sort']

        if len(query['repo']) > 0:
            es_query['repo'] = query['repo']

        if query.get('aggs'):
            es_query['aggs'] = query['aggs']

        return es_query



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.tests import TestCase

from analytics import init_app
from analytics.chart_config import ChartConfig


class ChartConfigTestCase(TestCase):
    def setUp(self):
        self.maxDiff = None

        with self.app.app_context():
            init_app(self.app)

            self.app.data.insert('vocabularies', [{
                '_id': 'categories',
                'items': [
                    {'qcode': 'a', 'name': 'Advisories', 'is_active': True},
                    {'qcode': 'b', 'name': 'Basketball', 'is_active': True},
                    {'qcode': 'c', 'name': 'Cricket', 'is_active': True}
                ]
            }, {
                '_id': 'urgency',
                'items': [
                    {'qcode': 1, 'name': 1, 'is_active': True},
                    {'qcode': 2, 'name': 2, 'is_active': True},
                    {'qcode': 3, 'name': 3, 'is_active': True},
                    {'qcode': 4, 'name': 4, 'is_active': True},
                    {'qcode': 5, 'name': 5, 'is_active': True}
                ]
            }, {
                '_id': 'genre',
                'items': [
                    {'qcode': 'Article', 'name': 'Article (news)', 'is_active': True},
                    {'qcode': 'Sidebar', 'name': 'Sidebar', 'is_active': True},
                    {'qcode': 'Factbox', 'name': 'Factbox', 'is_active': True}
                ],
            }])

            self.app.data.insert('desks', [
                {'_id': 'desk1', 'name': 'Politic Desk'},
                {'_id': 'desk2', 'name': 'Sports Desk'},
                {'_id': 'desk3', 'name': 'System Desk'}
            ])

            self.app.data.insert('users', [
                {'_id': 'user1', 'display_name': 'first user', 'is_active': True, 'is_enabled': True},
                {'_id': 'user2', 'display_name': 'second user', 'is_active': True, 'is_enabled': True},
                {'_id': 'user3', 'display_name': 'last user', 'is_active': True, 'is_enabled': True}
            ])

    @staticmethod
    def _gen_single_chart(chart_id, chart_type):
        chart = ChartConfig(chart_id, chart_type)
        chart.add_source('anpa_category.qcode', {'a': 3, 'b': 4, 'c': 1})
        return chart

    @staticmethod
    def _gen_stacked_chart(chart_id, chart_type):
        chart = ChartConfig(chart_id, chart_type)
        chart.add_source('anpa_category.qcode', {
            'a': {1: 1, 3: 1},
            'b': {1: 1, 3: 2},
            'c': {1: 2, 3: 1, 5: 1},
        })
        chart.add_source('urgency', {1: 4, 3: 4, 5: 1})
        return chart

    def test_generate_single_series(self):
        chart = self._gen_single_chart('cid', 'bar')
        chart.title = 'Charts'
        chart.subtitle = 'For Today'

        self.assertFalse(chart.is_multi_source())
        self.assertEqual(chart.gen_config(), {
            'id': 'cid',
            'type': 'highcharts',
            'chart': {'zoomType': 'y'},
            'title': {'text': 'Charts'},
            'subtitle': {'text': 'For Today'},
            'xAxis': [{
                'title': {'text': 'Category'},
                'categories': ['Basketball', 'Advisories', 'Cricket'],
                'allowDecimals': False,
                'type': 'category'
            }],
            'yAxis': [{
                'title': {'text': 'Published Stories'},
                'stackLabels': {'enabled': False},
                'allowDecimals': False,
            }],
            'legend': {'enabled': False},
            'tooltip': {
                'headerFormat': '{point.x}: {point.y}',
                'pointFormat': '',
            },
            'plotOptions': {
                'series': {'dataLabels': {'enabled': True}},
                'bar': {'colorByPoint': True},
                'column': {'colorByPoint': True},
            },
            'series': [{
                'name': 'Category',
                'data': [4, 3, 1],
                'type': 'bar',
                'xAxis': 0,

            }],
            'credits': {'enabled': False},
            'fullHeight': False,
            'time': {'useUTC': True}
        })

    def test_sort_single_series(self):
        chart = self._gen_single_chart('cid', 'bar')
        chart.sort_order = 'asc'

        config = chart.gen_config()
        self.assertEqual(config['xAxis'], [{
            'title': {'text': 'Category'},
            'categories': ['Cricket', 'Advisories', 'Basketball'],
            'allowDecimals': False,
            'type': 'category'
        }])
        self.assertEqual(config['series'], [{
            'name': 'Category',
            'data': [1, 3, 4],
            'type': 'bar',
            'xAxis': 0
        }])

        chart.sort_order = 'desc'
        config = chart.gen_config()
        self.assertEqual(config['xAxis'], [{
            'title': {'text': 'Category'},
            'categories': ['Basketball', 'Advisories', 'Cricket'],
            'allowDecimals': False,
            'type': 'category'
        }])
        self.assertEqual(config['series'], [{
            'name': 'Category',
            'data': [4, 3, 1],
            'type': 'bar',
            'xAxis': 0,
        }])

    def test_generate_stacked_series(self):
        chart = self._gen_stacked_chart('cid', 'column')
        chart.title = 'Charts'
        chart.subtitle = 'For Today'

        self.assertTrue(chart.is_multi_source())
        self.assertEqual(chart.gen_config(), {
            'id': 'cid',
            'type': 'highcharts',
            'chart': {'zoomType': 'x'},
            'title': {'text': 'Charts'},
            'subtitle': {'text': 'For Today'},
            'xAxis': [{
                'title': {'text': 'Category'},
                'categories': ['Cricket', 'Basketball', 'Advisories'],
                'allowDecimals': False,
                'type': 'category'
            }],
            'yAxis': [{
                'title': {'text': 'Published Stories'},
                'stackLabels': {'enabled': True},
                'allowDecimals': False,
            }],
            'legend': {
                'enabled': True,
                'title': {'text': 'Urgency'},
            },
            'tooltip': {
                'headerFormat': '{series.name}/{point.x}: {point.y}',
                'pointFormat': '',
            },
            'plotOptions': {
                'series': {'dataLabels': {'enabled': False}},
                'bar': {'colorByPoint': False},
                'column': {'colorByPoint': False},
            },
            'series': [{
                'name': '1',
                'data': [2, 1, 1],
                'type': 'column',
                'xAxis': 0,
                'stack': 0,
                'stacking': 'normal'
            }, {
                'name': '3',
                'data': [1, 2, 1],
                'type': 'column',
                'xAxis': 0,
                'stack': 0,
                'stacking': 'normal'
            }, {
                'name': '5',
                'data': [1, 0, 0],
                'type': 'column',
                'xAxis': 0,
                'stack': 0,
                'stacking': 'normal'
            }],
            'credits': {'enabled': False},
            'fullHeight': False,
            'time': {'useUTC': True}
        })

    def test_sort_stacked_series(self):
        chart = self._gen_stacked_chart('cid', 'bar')
        chart.sort_order = 'asc'

        config = chart.gen_config()
        self.assertEqual(config['xAxis'], [{
            'title': {'text': 'Category'},
            'categories': ['Advisories', 'Basketball', 'Cricket'],
            'allowDecimals': False,
            'type': 'category'
        }])
        self.assertEqual(config['series'], [{
            'name': '1',
            'data': [1, 1, 2],
            'type': 'bar',
            'xAxis': 0,
            'stack': 0,
            'stacking': 'normal'
        }, {
            'name': '3',
            'data': [1, 2, 1],
            'type': 'bar',
            'xAxis': 0,
            'stack': 0,
            'stacking': 'normal'
        }, {
            'name': '5',
            'data': [0, 0, 1],
            'type': 'bar',
            'xAxis': 0,
            'stack': 0,
            'stacking': 'normal'
        }])

        chart.sort_order = 'desc'
        config = chart.gen_config()
        self.assertEqual(config['xAxis'], [{
            'title': {'text': 'Category'},
            'categories': ['Cricket', 'Basketball', 'Advisories'],
            'allowDecimals': False,
            'type': 'category'
        }])
        self.assertEqual(config['series'], [{
            'name': '1',
            'data': [2, 1, 1],
            'type': 'bar',
            'xAxis': 0,
            'stack': 0,
            'stacking': 'normal'
        }, {
            'name': '3',
            'data': [1, 2, 1],
            'type': 'bar',
            'xAxis': 0,
            'stack': 0,
            'stacking': 'normal'
        }, {
            'name': '5',
            'data': [1, 0, 0],
            'type': 'bar',
            'xAxis': 0,
            'stack': 0,
            'stacking': 'normal'
        }])

    def test_generate_single_column_table(self):
        chart = self._gen_single_chart('tid', 'table')
        chart.title = 'Tables'
        chart.subtitle = 'For Today'

        self.assertFalse(chart.is_multi_source())
        self.assertEqual(chart.gen_config(), {
            'id': 'tid',
            'type': 'table',
            'chart': {'type': 'column'},
            'title': 'Tables',
            'subtitle': 'For Today',
            'xAxis': [{
                'title': {'text': 'Category'},
                'categories': ['Basketball', 'Advisories', 'Cricket'],
                'allowDecimals': False,
                'type': 'category'
            }],
            'series': [{
                'name': 'Category',
                'data': [4, 3, 1],
                'type': 'column',
                'xAxis': 0,
            }],
            'headers': ['Category', 'Published Stories'],
            'rows': [
                ['Basketball', 4],
                ['Advisories', 3],
                ['Cricket', 1]
            ],
        })

    def test_sort_single_column_table(self):
        chart = self._gen_single_chart('tid', 'table')
        chart.sort_order = 'asc'

        config = chart.gen_config()
        self.assertEqual(config['xAxis'], [{
            'title': {'text': 'Category'},
            'categories': ['Cricket', 'Advisories', 'Basketball'],
            'allowDecimals': False,
            'type': 'category'
        }])
        self.assertEqual(config['series'], [{
            'name': 'Category',
            'data': [1, 3, 4],
            'type': 'column',
            'xAxis': 0
        }])
        self.assertEqual(config['rows'], [
            ['Cricket', 1],
            ['Advisories', 3],
            ['Basketball', 4]
        ])

        chart.sort_order = 'desc'
        config = chart.gen_config()
        self.assertEqual(config['xAxis'], [{
            'title': {'text': 'Category'},
            'categories': ['Basketball', 'Advisories', 'Cricket'],
            'allowDecimals': False,
            'type': 'category'
        }])
        self.assertEqual(config['series'], [{
            'name': 'Category',
            'data': [4, 3, 1],
            'type': 'column',
            'xAxis': 0
        }])
        self.assertEqual(config['rows'], [
            ['Basketball', 4],
            ['Advisories', 3],
            ['Cricket', 1]
        ])

    def test_generate_multi_column_table(self):
        chart = self._gen_stacked_chart('tid', 'table')
        chart.title = 'Tables'
        chart.subtitle = 'For Today'

        self.assertTrue(chart.is_multi_source())
        self.assertEqual(chart.gen_config(), {
            'id': 'tid',
            'type': 'table',
            'chart': {'type': 'column'},
            'title': 'Tables',
            'subtitle': 'For Today',
            'xAxis': [{
                'title': {'text': 'Category'},
                'categories': ['Cricket', 'Basketball', 'Advisories'],
                'allowDecimals': False,
                'type': 'category'
            }],
            'series': [{
                'name': '1',
                'data': [2, 1, 1],
                'type': 'column',
                'xAxis': 0,
                'stack': 0,
                'stacking': 'normal'
            }, {
                'name': '3',
                'data': [1, 2, 1],
                'type': 'column',
                'xAxis': 0,
                'stack': 0,
                'stacking': 'normal'
            }, {
                'name': '5',
                'data': [1, 0, 0],
                'type': 'column',
                'xAxis': 0,
                'stack': 0,
                'stacking': 'normal'
            }],
            'headers': ['Category', '1', '3', '5', 'Total Stories'],
            'rows': [
                ['Cricket', 2, 1, 1, 4],
                ['Basketball', 1, 2, 0, 3],
                ['Advisories', 1, 1, 0, 2],
            ],
        })

    def test_sort_multi_column_table(self):
        chart = self._gen_stacked_chart('tid', 'table')
        chart.sort_order = 'asc'

        config = chart.gen_config()
        self.assertEqual(config['xAxis'], [{
            'title': {'text': 'Category'},
            'categories': ['Advisories', 'Basketball', 'Cricket'],
            'allowDecimals': False,
            'type': 'category'
        }])
        self.assertEqual(config['series'], [{
            'name': '1',
            'data': [1, 1, 2],
            'type': 'column',
            'xAxis': 0,
            'stack': 0,
            'stacking': 'normal'
        }, {
            'name': '3',
            'data': [1, 2, 1],
            'type': 'column',
            'xAxis': 0,
            'stack': 0,
            'stacking': 'normal'
        }, {
            'name': '5',
            'data': [0, 0, 1],
            'type': 'column',
            'xAxis': 0,
            'stack': 0,
            'stacking': 'normal'
        }])
        self.assertEqual(config['rows'], [
            ['Advisories', 1, 1, 0, 2],
            ['Basketball', 1, 2, 0, 3],
            ['Cricket', 2, 1, 1, 4],
        ])

        chart.sort_order = 'desc'
        config = chart.gen_config()
        self.assertEqual(config['xAxis'], [{
            'title': {'text': 'Category'},
            'categories': ['Cricket', 'Basketball', 'Advisories'],
            'allowDecimals': False,
            'type': 'category'
        }])
        self.assertEqual(config['series'], [{
            'name': '1',
            'data': [2, 1, 1],
            'type': 'column',
            'xAxis': 0,
            'stack': 0,
            'stacking': 'normal'
        }, {
            'name': '3',
            'data': [1, 2, 1],
            'type': 'column',
            'xAxis': 0,
            'stack': 0,
            'stacking': 'normal'
        }, {
            'name': '5',
            'data': [1, 0, 0],
            'type': 'column',
            'xAxis': 0,
            'stack': 0,
            'stacking': 'normal'
        }])
        self.assertEqual(config['rows'], [
            ['Cricket', 2, 1, 1, 4],
            ['Basketball', 1, 2, 0, 3],
            ['Advisories', 1, 1, 0, 2],
        ])

    def test_translate_anpa_category(self):
        chart = ChartConfig('catrgory', 'bar')
        chart.add_source('anpa_category.qcode', {'a': 3, 'b': 4, 'c': 1})

        self.assertEqual(chart.translations, {})

        chart.load_translations()
        self.assertEqual(chart.translations, {
            'anpa_category_qcode': {
                'title': 'Category',
                'names': {
                    'a': 'Advisories',
                    'b': 'Basketball',
                    'c': 'Cricket'
                }
            }
        })

    def test_translate_urgency(self):
        chart = ChartConfig('urgency', 'bar')
        chart.add_source('urgency', {1: 4, 3: 4, 5: 1})

        self.assertEqual(chart.translations, {})

        chart.load_translations()
        self.assertEqual(chart.translations, {
            'urgency': {
                'title': 'Urgency',
                'names': {
                    1: 1,
                    2: 2,
                    3: 3,
                    4: 4,
                    5: 5
                }
            }
        })

    def test_translate_genre(self):
        chart = ChartConfig('genre', 'bar')
        chart.add_source('genre.qcode', {'Article': 4, 'Sidebar': 5, 'Factbox': 1})

        self.assertEqual(chart.translations, {})

        chart.load_translations()
        self.assertEqual(chart.translations, {
            'genre_qcode': {
                'title': 'Genre',
                'names': {
                    'Article': 'Article (news)',
                    'Sidebar': 'Sidebar',
                    'Factbox': 'Factbox'
                }
            }
        })

    def test_translate_desk(self):
        chart = ChartConfig('desk', 'bar')
        chart.add_source('task.desk', {'desk1': 4, 'desk2': 5, 'desk3': 1})

        self.assertEqual(chart.translations, {})

        chart.load_translations()
        self.assertEqual(chart.translations, {
            'task_desk': {
                'title': 'Desk',
                'names': {
                    'desk1': 'Politic Desk',
                    'desk2': 'Sports Desk',
                    'desk3': 'System Desk'
                }
            }
        })

    def test_translate_user(self):
        chart = ChartConfig('user', 'bar')
        chart.add_source('task.user', {'user1': 3, 'user2': 4, 'user3': 5})

        self.assertEqual(chart.translations, {})

        chart.load_translations()
        self.assertEqual(chart.translations, {
            'task_user': {
                'title': 'User',
                'names': {
                    'user1': 'first user',
                    'user2': 'second user',
                    'user3': 'last user'
                }
            }
        })

    def test_translate_authors(self):
        chart = ChartConfig('author', 'bar')
        chart.add_source('authors.parent', {'user1': 3, 'user2': 4, 'user3': 5})

        self.assertEqual(chart.translations, {})

        chart.load_translations()
        self.assertEqual(chart.translations, {
            'authors_parent': {
                'title': 'Author',
                'names': {
                    'user1': 'first user',
                    'user2': 'second user',
                    'user3': 'last user'
                }
            }
        })

    def test_translate_state(self):
        chart = ChartConfig('state', 'bar')
        chart.add_source('state', {'published': 3, 'killed': 1, 'updated': 5})

        self.assertEqual(chart.translations, {})

        chart.load_translations()
        self.assertEqual(chart.translations, {
            'state': {
                'title': 'State',
                'names': {
                    'published': 'Published',
                    'killed': 'Killed',
                    'corrected': 'Corrected',
                    'updated': 'Updated',
                }
            }
        })

    def test_translate_source(self):
        chart = ChartConfig('source', 'bar')
        chart.add_source('source', {'aap': 3, 'ftp': 1, 'ap': 5})

        self.assertEqual(chart.translations, {})

        chart.load_translations()
        self.assertEqual(chart.translations, {
            'source': {
                'title': 'Source',
                'names': {}
            }
        })



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013-2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource

from analytics.stats.stats_report_service import StatsReportService
from analytics.chart_config import SDChart, ChartConfig
from analytics.common import seconds_to_human_readable


class ProductionTimeReportResource(Resource):
    """Desk Activity Report schema"""

    item_methods = ['GET']
    resource_methods = ['GET']
    privileges = {'GET': 'production_time_report'}


class ProductionTimeReportService(StatsReportService):
    aggregations = {
        'operations': {
            'terms': {
                'field': 'stats.timeline.operation',
                'size': 0
            }
        }
    }
    histogram_source_field = 'stats.timeline.operation_created'
    date_filter_field = 'versioncreated'

    def get_request_aggregations(self, params, args):
        params = args.get('params') or {}
        lt, gte, time_zone = self._es_get_date_filters(params)

        return {
            'inner': {
                'nested': {'path': 'stats.desk_transitions'},
                'aggs': {
                    'date_filter': {
                        'filter': {
                            'range': {
                                'stats.desk_transitions.entered': {
                                    'gte': gte,
                                    'lt': lt,
                                    'time_zone': time_zone
                                }
                            }
                        },
                        'aggs': {
                            'desks': {
                                'terms': {
                                    'field': 'stats.desk_transitions.desk',
                                    'size': 0
                                },
                                'aggs': {
                                    'stats': {
                                        'stats': {
                                            'field': 'stats.desk_transitions.duration'
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

    def generate_report(self, docs, args):
        aggregations = getattr(docs, 'hits', {}).get('aggregations') or {}

        date_filter = (aggregations.get('inner') or {}).get('date_filter') or {}
        desk_buckets = (date_filter.get('desks') or {}).get('buckets') or []

        if len(desk_buckets) < 1:
            return {}

        report = {'desk_stats': {}}

        for bucket in desk_buckets:
            desk_id = bucket.get('key')

            if not desk_id:
                continue

            stats = bucket.get('stats') or {}

            report['desk_stats'][desk_id] = {
                'count': stats.get('count') or 0,
                'min': stats.get('min') or 0,
                'max': stats.get('max') or 0,
                'avg': stats.get('avg') or 0,
                'sum': stats.get('sum') or 0
            }

        return report

    def generate_highcharts_config(self, docs, args):
        params = args.get('params') or {}
        chart_params = params.get('chart') or {}
        report = self.generate_report(docs, args)

        stats = params.get('stats') or {}
        desk_stats = report.get('desk_stats') or {}
        desk_ids = list(desk_stats.keys())
        stat_types = [
            stat
            for stat in ['sum', 'max', 'avg', 'min']
            if stats.get(stat)
        ]
        sort_order = chart_params.get('sort_order') or 'desc'

        def get_sum_stats(desk_id):
            return sum([
                value
                for stat, value in desk_stats[desk_id].items()
                if stat in stat_types
            ])

        sorted_desk_ids = [
            desk_id
            for desk_id in sorted(
                desk_ids,
                key=lambda kv: get_sum_stats(kv) if sort_order == 'asc'
                else -get_sum_stats(kv)
            )
        ]

        if chart_params.get('subtitle'):
            subtitle = chart_params['subtitle']
        else:
            subtitle = ChartConfig.gen_subtitle_for_dates(params)

        chart = SDChart.Chart(
            'production_time_report',
            chart_type='highcharts',
            title=chart_params.get('title') or 'Production Times',
            subtitle=subtitle,
            default_config=ChartConfig.defaultConfig,
            colour_by_point=False,
            data_labels=True,
            legend_title='Production Time',
            tooltip_header='{series.name}/{point.x}: {point.y}',
            translations=ChartConfig.get_translations(['task.desk'])
        )

        chart.set_translation('production_stats', 'Production Stats', {
            'min': 'Minimum',
            'sum': 'Sum',
            'avg': 'Average',
            'max': 'Maximum'
        })

        axis = chart.add_axis().set_options(
            type='category',
            default_chart_type=chart_params.get('type') or 'column',
            y_title='Seconds spent producing content',
            category_field='task.desk',
            categories=sorted_desk_ids,
            stack_labels=False
        )

        def gen_data_entry(stat):
            return {
                'dataLabels': {
                    'enabled': True,
                    'format': seconds_to_human_readable(stat)
                },
                'y': stat
            }

        for stat_type in stat_types:
            axis.add_series().set_options(
                field='production_stats',
                name=stat_type,
                stack=0,
                stack_type='normal',
                data=[
                    gen_data_entry((desk_stats.get(desk_id) or {}).get(stat_type) or 0)
                    for desk_id in sorted_desk_ids
                ]
            )

        report['highcharts'] = [chart.gen_config()]

        return report



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from .featuremedia_updates_report import FeaturemdiaUpdatesReportResource,\
    FeaturemediaUpdatesTimeReportService
from analytics.common import register_report


def init_app(app):
    # Don't register this endpoint if archive stats aren't being generated
    # Generating stats with PUBLISH_ASSOCIATED_ITEMS=True is currently not supported
    if not app.config.get('ANALYTICS_ENABLE_ARCHIVE_STATS', False) or \
            app.config.get('PUBLISH_ASSOCIATED_ITEMS', False):
        return

    endpoint_name = 'featuremedia_updates_report'
    service = FeaturemediaUpdatesTimeReportService(endpoint_name, backend=superdesk.get_backend())
    FeaturemdiaUpdatesReportResource(endpoint_name, app=app, service=service)

    register_report('featuremedia_updates_report', 'featuremedia_updates_report')

    superdesk.privilege(
        name='featuremedia_updates_report',
        label='Analytics - Featuremdia Updates Report',
        description='User can view Featuremdia Updates Report'
    )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
#  Copyright 2013-2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk import Command, command, get_resource_service, Option
from superdesk.logging import logger
from superdesk.utc import utcnow
from superdesk.metadata.item import ITEM_STATE, ITEM_TYPE, FORMAT, SCHEDULE_SETTINGS, \
    CONTENT_STATE, PUBLISH_SCHEDULE, EMBARGO, BYLINE
from superdesk.text_utils import get_par_count
from superdesk.celery_task_utils import get_lock_id
from superdesk.lock import lock, unlock
from superdesk.signals import signals

from analytics.stats.common import STAT_TYPE, OPERATION
from analytics.stats import desk_transitions

from eve.utils import config
from copy import deepcopy
from datetime import timedelta

gen_stats_signals = {
    'start': signals.signal('gen_archive_statistics:start'),
    'generate': signals.signal('gen_archive_statistics:generate'),
    'init_timeline': signals.signal('gen_archive_statistics:init_timeline'),
    'process': signals.signal('gen_archive_statistics:process'),
    'complete': signals.signal('gen_archive_statistics:complete'),
    'finish': signals.signal('gen_archive_statistics:finish'),
}


def connect_stats_signals(
    on_start=None,
    on_generate=None,
    on_init_timeline=None,
    on_process=None,
    on_complete=None,
    on_finish=None
):
    """Connect functions to the gen_stats_signals

    :param on_start: Callback for start signal
    :param on_generate: Callback for generate signal
    :param on_init_timeline: Callback for init_timeline signal
    :param on_process: Callback for process signal
    :param on_complete: Callback for complete signal
    :param on_finish: Callback for finish signal
    """
    if on_start:
        gen_stats_signals['start'].connect(on_start)

    if on_generate:
        gen_stats_signals['generate'].connect(on_generate)

    if on_init_timeline:
        gen_stats_signals['init_timeline'].connect(on_init_timeline)

    if on_process:
        gen_stats_signals['process'].connect(on_process)

    if on_complete:
        gen_stats_signals['complete'].connect(on_complete)

    if on_finish:
        gen_stats_signals['finish'].connect(on_finish)


class GenArchiveStatistics(Command):
    """Generate statistics for archive documents based on archive_history documents

    Generates a linear timeline of operations based on documents from archive_history.
    This data is then used to search and aggregate for usage in generating charts.

    Options
    ::

        -d, --max-days (defaults to 3):
        Maximum number of days to process from archive_history
        -i, --item-id (defaults to None):
        Generate statistics for a single archive item only
        -c, --chunk-size (defaults to 1000):
        Number of archive history items to process per iteration

    If the config option ANALYTICS_ENABLE_ARCHIVE_STATS is true, this command will run in
    celery on a schedule every hour (minute=0).

    Example:
    ::

        $ python manage.py analytics:gen_archive_statistics
        $ python manage.py analytics:gen_archive_statistics -d 1
        $ python manage.py analytics:gen_archive_statistics -max-days 1
        $ python manage.py analytics:gen_archive_statistics -i 'id-of-item-to-gen-stats-for'
        $ python manage.py analytics:gen_archive_statistics -item-id 'id-of-item-to-gen-stats-for'
        $ python manage.py analytics:gen_archive_statistics -c 500
        $ python manage.py analytics:gen_archive_statistics -chunk-size 500

    There are 4 signals that are sent when generating statistics.
    This allows custom statistics to be generated and stored in the item.
    There is a dictionary in the schema for custom stats to be stored under the 'extra' attribute.

    Example:
    ::

        from analytics.stats.gen_archive_statistics import connect_stats_signals

        connect_stats_signals(on_generate, on_init_timeline, on_process, on_complete)
        # or by key values
        connect_stats_signals(on_process=on_process_stats)

    """

    option_list = [
        Option('--max-days', '-d', dest='max_days', default=3),
        Option('--item-id', '-i', dest='item_id', default=None),
        Option('--chunk-size', '-c', dest='chunk_size', default=1000)
    ]

    def run(self, max_days=3, item_id=None, chunk_size=1000):
        now_utc = utcnow()

        # If we're generating stats for a single item, then
        # don't set max_days, as we want to process all history records
        # for the provided item
        if item_id is not None:
            max_days = 0

        try:
            max_days = float(max_days)
        except (ValueError, TypeError):
            max_days = 3
        gte = None if max_days <= 0.0 else utcnow() - timedelta(days=max_days)

        try:
            chunk_size = int(chunk_size)
        except (ValueError, TypeError):
            chunk_size = 1000
        chunk_size = None if chunk_size <= 0 else chunk_size

        logger.info(
            'Starting to generate archive statistics: {}. gte={}. item_id={}. chunk_size={}'
            .format(
                now_utc,
                gte,
                item_id,
                chunk_size
            )
        )

        lock_name = get_lock_id('analytics', 'gen_archive_statistics')
        if not lock(lock_name, expire=610):
            logger.info('Generate archive statistics task is already running.')
            return

        items_processed = 0
        failed_ids = []
        num_history_items = 0

        try:
            items_processed, failed_ids, num_history_items = self.generate_stats(
                item_id,
                gte,
                chunk_size
            )
        except Exception:
            logger.exception('Failed to generate archive stats')
        finally:
            unlock(lock_name)

        if len(failed_ids) > 0:
            logger.warning('Failed to generate stats for items {}'.format(
                ', '.join(failed_ids)
            ))

        duration = (utcnow() - now_utc).total_seconds()
        logger.info(
            'Finished generating stats for {} items ({} history entries). Duration: {} seconds'
            .format(
                items_processed,
                num_history_items,
                int(duration)
            )
        )

    def generate_stats(self, item_id, gte, chunk_size):
        items_processed = 0
        failed_ids = []
        num_history_items = 0

        statistics_service = get_resource_service('archive_statistics')

        # Get the system record from the last run
        # This document stores the id of the last processed archive_history item
        last_history = statistics_service.get_last_run()
        last_entry_id = last_history.get('guid') or None

        if last_history.get('guid'):
            logger.info('Found previous run, continuing from history item {}'.format(
                last_history['guid']
            ))

        iterated_started = utcnow()
        for history_items in statistics_service.get_history_items(
                last_entry_id,
                gte,
                item_id,
                chunk_size
        ):
            if len(history_items) < 1:
                logger.info('No more history records to process')
                break

            gen_stats_signals['start'].send(self)

            num_history_items += len(history_items)
            last_entry_id = history_items[-1].get(config.ID_FIELD)

            items = self.gen_history_timelines(history_items)
            items_processed += len(items)
            self.process_timelines(items, failed_ids)

            time_diff = (utcnow() - iterated_started).total_seconds()
            logger.info('Processed {}/{} history/item records ({}/{} total) in {} seconds'.format(
                len(history_items),
                len(items),
                num_history_items,
                items_processed,
                int(time_diff)
            ))

            gen_stats_signals['finish'].send(self)
            iterated_started = utcnow()

        # Don't store the last processed id if we're generating stats for a single item
        if not item_id:
            # Create/Update the system record from this run
            # Storing the id of the last processed archive_history item
            statistics_service.set_last_run_id(last_entry_id, last_history)

        return items_processed, failed_ids, num_history_items

    def gen_history_timelines(self, history_items):
        items = {}

        statistics_service = get_resource_service('archive_statistics')

        def add_item(entry_id):
            if items.get(entry_id):
                return

            item = statistics_service.find_one(req=None, _id=str(entry_id)) or {}

            if not item.get('stats'):
                item['stats'] = {}

            items[entry_id] = {
                'item': item,
                '_id': entry_id,
                'updates': {'stats': {}}
            }

            items[entry_id]['updates'].update(item)

            def _set_timeline_processed(entry):
                entry['_processed'] = True
                return entry

            items[entry_id]['updates']['stats'].update({
                STAT_TYPE.TIMELINE: [
                    _set_timeline_processed(entry)
                    for entry in item['stats'].get(STAT_TYPE.TIMELINE) or []
                ]
            })

        for history_item in history_items:
            item_id = history_item.get('item_id')

            if not history_item.get('update'):
                history_item['update'] = {}

            try:
                add_item(item_id)
                self.gen_archive_stats_from_history(
                    items[item_id],
                    history_item
                )
            except Exception:
                logger.exception('Failed to process archive_history for item:{} history:{}'.format(
                    item_id,
                    history_item.get(config.ID_FIELD)
                ))

        return items

    def gen_archive_stats_from_history(self, item, history):
        self.set_operation(history)

        task = history['update'].get('task') or {}
        entry = {
            'history_id': history.get(config.ID_FIELD),
            'operation': history.get('operation'),
            'operation_created': history.get('_created'),
            'task': {
                'user': history.get('user_id'),
                'desk': task.get('desk'),
                'stage': task.get('stage'),
            },
            'update': history['update'],
            'version': history['version']
        }

        if history.get('original_item_id'):
            entry['original_item_id'] = history['original_item_id']

        if entry['operation'] != OPERATION.MOVE:
            item['updates']['stats'][STAT_TYPE.TIMELINE].append(entry)
        else:
            # Remove the new task details from the MOVE_FROM operation
            # Task will later be calculated from the previous history item
            task = entry.pop('task', {
                'user': history.get('user_id'),
                'desk': None,
                'stage': None
            })

            entry['task'] = {
                'user': history.get('user_id'),
                'desk': None,
                'stage': None
            }
            entry['operation'] = OPERATION.MOVE_FROM
            item['updates']['stats'][STAT_TYPE.TIMELINE].append(entry)

            # Copy the original history entry for the MOVE_TO entry
            # Assign the new task details to this entry
            entry = deepcopy(entry)
            entry['operation'] = OPERATION.MOVE_TO
            entry['task'] = task
            entry['_auto_generated'] = True
            item['updates']['stats'][STAT_TYPE.TIMELINE].append(entry)

    def set_operation(self, history):
        if history['operation'] == OPERATION.UPDATE and \
                history['update'].get('operation') == OPERATION.DESCHEDULE:
            history['operation'] = OPERATION.DESCHEDULE
        elif history['operation'] == OPERATION.PUBLISH:
            if history['update'].get(ITEM_STATE) != CONTENT_STATE.PUBLISHED and \
                    history['update'].get(PUBLISH_SCHEDULE):
                history['operation'] = OPERATION.PUBLISH_SCHEDULED
            elif history['update'].get(ITEM_STATE) == CONTENT_STATE.PUBLISHED and \
                    history['update'].get(EMBARGO):
                history['operation'] = OPERATION.PUBLISH_EMBARGO

    def set_metadata_updates(self, item, history):
        fields = [
            'original_creator', 'version_creator'
            'versioncreated', 'firstpublished', 'firstcreated',
            'source', 'original_source', 'ingest_provider',
            'anpa_category', 'subject', 'genre', 'company_codes',
            ITEM_TYPE, 'abstract', 'headline', 'slugline',
            'anpa_take_key', 'keywords', 'word_count', 'priority',
            'urgency', ITEM_STATE, 'pubstatus', 'flags', 'sms_message',
            FORMAT, 'auto_publish', 'assignment_id', 'rewrite_of', 'rewritten_by',
            'original_id', SCHEDULE_SETTINGS, 'task', PUBLISH_SCHEDULE, EMBARGO,
            'unique_id', 'unique_name', 'ingest_id', 'family_id', 'usageterms',
            'copyrightnotice', 'copyrightholder', 'profile', BYLINE, 'ednote',
            'dateline', 'expiry', 'place', 'template'
        ]

        # Calculate and store item attributes that the history service removes
        user_id = history['task']['user']
        created = history['operation_created']

        if 'version' in history:
            item['updates']['version'] = history['version']
            item['updates']['version_creator'] = user_id
            item['updates']['versioncreated'] = created

        if 'original_creator' not in item['updates']:
            item['updates']['original_creator'] = user_id
            item['updates']['firstcreated'] = created

        for field in fields:
            if field in history['update']:
                item['updates'][field] = history['update'][field]

    def process_timelines(self, items, failed_ids):
        statistics_service = get_resource_service('archive_statistics')
        items_to_create = []
        rewrites = []

        for item_id, item in items.items():
            try:
                self.gen_stats_from_timeline(item)
            except Exception:
                logger.exception('Failed to generate stats for item {}'.format(item_id))
                failed_ids.append(item_id)
                continue

            if item['updates'].get('rewrite_of') and \
                    (item['updates'].get('time_to_first_publish') or 0) > 0:
                rewrites.append(item_id)

            if not item['item'].get(config.ID_FIELD):
                item['updates'][config.ID_FIELD] = item_id
                item['updates']['stats_type'] = 'archive'
                items_to_create.append(item['updates'])
            else:
                try:
                    statistics_service.patch(
                        item_id,
                        item['updates']
                    )
                except Exception:
                    logger.exception('Failed to update stats for item {}. updates={}'.format(
                        item_id,
                        item.get('updates')
                    ))
                    failed_ids.append(item_id)

        if len(items_to_create) > 0:
            try:
                statistics_service.post(items_to_create)
            except Exception:
                item_ids = [item.get(config.ID_FIELD) for item in items_to_create]
                logger.exception('Failed to create stat entries for items {}'.format(
                    ', '.join(item_ids)
                ))
                failed_ids.extend(failed_ids)

        for item_id in rewrites:
            item = items[item_id]

            updated_at = item['updates'].get('firstpublished')
            if not updated_at:
                logger.warning('Failed {}, updated_at not defined'.format(item_id))
                continue

            original_id = item['updates'].get('rewrite_of')
            if not original_id:
                logger.warning('Failed {}, original_id not defined'.format(item_id))
                continue

            original = statistics_service.find_one(req=None, _id=original_id)
            if not original:
                logger.warning('Failed {}, original not found'.format(item_id))
                continue

            published_at = original.get('firstpublished')
            if not published_at:
                logger.warning('Failed {}, published_at not defined'.format(original_id))
                continue

            statistics_service.patch(
                original_id,
                {'time_to_next_update_publish': (updated_at - published_at).total_seconds()}
            )

    def _store_update_fields(self, entry):
        update = {}

        if entry.get('operation') in [OPERATION.ITEM_LOCK, OPERATION.ITEM_UNLOCK]:
            # If the entry belongs to a lock, store the lock information
            update = entry.get('update') or {}

        desk_transitions.store_update_fields(entry, update)
        gen_stats_signals['generate'].send(self, entry=entry, update=update)

        if update:
            entry['update'] = update
        else:
            entry.pop('update', None)

    def gen_stats_from_timeline(self, item):
        item.setdefault('updates', {})
        updates = item['updates']

        updates.setdefault('stats', {})
        stats = updates['stats']

        if len(stats.get(STAT_TYPE.TIMELINE) or []) < 1:
            return

        new_timeline = []
        desk_transitions.init(stats)
        gen_stats_signals['init_timeline'].send(self, stats=stats)

        try:
            entries = sorted(
                stats[STAT_TYPE.TIMELINE],
                key=lambda k: (k['operation_created'], k['history_id'])
            )
        except Exception as e:
            logger.exception('Failed to sort timeline {}'.format(stats[STAT_TYPE.TIMELINE]))
            raise e

        # If the first history item has original_item_id attribute,
        # then this item is a duplicate of another item
        updates['_duplicate'] = entries[0].get('original_item_id')

        # Default the paragraph count to 0
        # We'll update this count while processing the timeline
        updates['par_count'] = 0

        for entry in entries:
            entry.setdefault('update', {})
            self.set_metadata_updates(item, entry)
            self.set_timeline_entry_task_details(entry, updates)

            if self.skip_timeline_entry(entry, updates):
                continue

            # Remove the update attribute before adding to the timeline
            update = entry.get('update') or {}
            self._store_update_fields(entry)

            # Update the paragraph count from this history entry
            self.update_par_count_from_timeline_entry(entry, updates, update)

            new_timeline.append(entry)

            # Use a copy of entry after adding to the timeline
            # So that any changes from here do not modify the existing timeline entry
            entry = deepcopy(entry)

            operation = entry.get('operation')
            operation_created = entry.get('operation_created')

            if operation == OPERATION.PUBLISH:
                updates['_published'] = True
                if not updates.get('firstpublished'):
                    updates['firstpublished'] = operation_created
            elif operation in [OPERATION.CREATE, OPERATION.FETCH] and \
                    not updates.get('firstcreated'):
                updates['firstcreated'] = operation_created

            desk_transitions.process(entry, new_timeline, updates, update, stats)
            gen_stats_signals['process'].send(
                self,
                entry=entry,
                new_timeline=new_timeline,
                updates=updates,
                update=update,
                stats=stats
            )

        desk_transitions.complete(stats, updates)
        gen_stats_signals['complete'].send(self, stats=stats, orig=item, updates=updates)

        if updates.get('firstpublished') and updates.get('firstcreated'):
            updates['time_to_first_publish'] = (
                updates['firstpublished'] - updates['firstcreated']
            ).total_seconds()

        def _remove_tmp_fields(entry):
            entry.pop('_processed', None)
            return entry

        stats[STAT_TYPE.TIMELINE] = [_remove_tmp_fields(entry) for entry in new_timeline]

        for key in list(updates.keys()):
            if key.startswith('_'):
                updates.pop(key)

    def set_timeline_entry_task_details(self, entry, updates):
        """Calculate the desk, stage and user for this entry"""

        # Store temporary attribute for storing last known task
        updates.setdefault('_last_task', {
            'user': None,
            'desk': None,
            'stage': None
        })

        for field in ['desk', 'stage', 'user']:
            # If the current history record has the task details,
            # then update the last known task
            if entry['task'].get(field):
                updates['_last_task'][field] = entry['task'][field]

            # If the task is not set in the history record
            # then update it from the last known task
            elif not entry['task'].get(field) and updates['_last_task'][field]:
                entry['task'][field] = updates['_last_task'][field]

    def skip_timeline_entry(self, entry, updates):
        """Filter out history that belongs to the parent item (duplicated items)"""

        # Store temporary attribute for storing ids of history records
        # that have already been processed (no duplicate history records in stats)
        updates.setdefault('_processed_ids', [])

        # Skip history records that belong to the parent item
        # (history records are copied for duplicate items)
        original_item_id = entry.pop('original_item_id', None)
        if updates['_duplicate'] and original_item_id is not None:
            return True

        # If a history ID is set, then make sure we don't add duplicate history entries
        history_id = entry.get('history_id') or None
        if history_id is not None and not entry.get('_auto_generated'):
            # Filter out duplicate archive_history entries
            if entry['history_id'] in updates['_processed_ids']:
                return True

            updates['_processed_ids'].append(entry['history_id'])

        return False

    def update_par_count_from_timeline_entry(self, entry, updates, update):
        """Generate and store the paragraph count from body_html"""

        if len(update.get('body_html') or '') > 0:
            entry['par_count'] = get_par_count(update['body_html'])
            updates['par_count'] = entry['par_count']
        else:
            entry['par_count'] = updates['par_count']

        if 'original_par_count' not in updates and entry['par_count'] > 0:
            updates['original_par_count'] = entry['par_count']


command('analytics:gen_archive_statistics', GenArchiveStatistics())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2016 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk

from analytics.report_configs import ReportConfigsResource, ReportConfigsService
from analytics.base_report import BaseReportService
from analytics.saved_reports import SavedReportsResource, SavedReportsService
from analytics.reports.scheduled_reports import ScheduledReportsResource, ScheduledReportsService
from analytics.content_publishing_report import init_app as init_content_publishing_report
from analytics.publishing_performance_report import init_app as init_publishing_performance_report
from analytics.email_report import init_app as init_email_report
from analytics.planning_usage_report import init_app as init_planning_usage_report
from analytics.stats import init_app as init_stats
from analytics.desk_activity_report import init_app as init_desk_activity_report
from analytics.production_time_report import init_app as init_production_time_report
from analytics.user_activity_report import init_app as init_user_acitivity_report
from analytics.featuremedia_updates_report import init_app as init_featuremedia_updates_report
from analytics.update_time_report import init_app as init_update_time_report

from analytics.commands import SendScheduledReports  # noqa
from analytics.common import is_highcharts_installed, register_report
from superdesk.celery_app import celery
from superdesk.default_settings import celery_queue, crontab

__version__ = '1.33.3'


def init_schedule_task(app):
    # Now check the application config to see if scheduled reports is enabled
    if not app.config.get('ANALYTICS_ENABLE_SCHEDULED_REPORTS', False):
        return

    # First check to see if highcharts-export-server is installed and globally accessible
    if not is_highcharts_installed():
        superdesk.logger.warn('Highcharts export server is not installed')
        return

    # Make sure the TASK_ROUTES are set
    if not app.config['CELERY_TASK_ROUTES']:
        app.config['CELERY_TASK_ROUTES'] = {}

    # If the celery task is not configured, then set the default now
    if not app.config['CELERY_TASK_ROUTES'].get('analytics.send_scheduled_reports'):
        app.config['CELERY_TASK_ROUTES']['analytics.send_scheduled_reports'] = {
            'queue': celery_queue('default'),
            'routing_key': 'analytics.schedules'
        }

    # Make sure the BEAT_SCHEDULE are set
    if not app.config['CELERY_BEAT_SCHEDULE']:
        app.config['CELERY_BEAT_SCHEDULE'] = {}

    # If the celery schedule is not configured, then set the default now
    if not app.config['CELERY_BEAT_SCHEDULE'].get('analytics:send_scheduled_reports'):
        app.config['CELERY_BEAT_SCHEDULE']['analytics:send_scheduled_reports'] = {
            'task': 'analytics.send_scheduled_reports',
            'schedule': crontab(minute='0')  # Runs once every hour
        }


def init_app(app):
    endpoint_name = 'scheduled_reports'
    service = ScheduledReportsService(endpoint_name, backend=superdesk.get_backend())
    ScheduledReportsResource(endpoint_name, app=app, service=service)

    endpoint_name = SavedReportsResource.endpoint_name
    service = SavedReportsService(endpoint_name, backend=superdesk.get_backend())
    SavedReportsResource(endpoint_name, app=app, service=service)

    superdesk.privilege(
        name='global_saved_reports',
        label='Analytics - Manage Global Saved Reports',
        description='User can manage other uses\' global saved reports'
    )
    superdesk.privilege(
        name='saved_reports',
        label='Analytics - Manage Saved Reports',
        description='User can manage saved reports'
    )
    superdesk.privilege(
        name='scheduled_reports',
        label='Analytics - Manage Scheduling Reports',
        description='User can manage scheduling of reports'
    )

    endpoint_name = ReportConfigsResource.endpoint_name
    service = ReportConfigsService(endpoint_name, backend=superdesk.get_backend())
    ReportConfigsResource(endpoint_name, app=app, service=service)

    init_content_publishing_report(app)
    init_publishing_performance_report(app)
    init_email_report(app)
    init_planning_usage_report(app)
    init_stats(app)
    init_desk_activity_report(app)
    init_production_time_report(app)
    init_user_acitivity_report(app)
    init_featuremedia_updates_report(app)
    init_update_time_report(app)

    app.client_config.setdefault('highcharts', {})
    app.client_config['highcharts']['license'] = {
        'id': app.config.get('HIGHCHARTS_LICENSE_ID') or '',
        'type': app.config.get('HIGHCHARTS_LICENSE_TYPE') or 'OEM',
        'licensee': app.config.get('HIGHCHARTS_LICENSEE') or '',
        'contact': app.config.get('HIGHCHARTS_LICENSEE_CONTACT') or '',
        'customer_id': app.config.get('HIGHCHARTS_LICENSE_CUSTOMER_ID') or '',
        'expiry': app.config.get('HIGHCHARTS_LICENSE_EXPIRY') or ''
    }

    # If this app is for testing, then create an endpoint for the base reporting service
    # so the core searching/aggregation functionality can be tested
    if app.testing:
        class TestReportService(BaseReportService):
            aggregations = {
                'category': {'terms': {'field': 'anpa_category.qcode', 'size': 0}},
                'source': {'terms': {'field': 'source', 'size': 0}}
            }

        class BaseReportResource(superdesk.resource.Resource):
            item_methods = ['GET']
            resource_methods = ['GET']

        endpoint_name = 'analytics_test_report'
        service = TestReportService(endpoint_name, backend=superdesk.get_backend())
        BaseReportResource(endpoint_name, app=app, service=service)

    init_schedule_task(app)


@celery.task(soft_time_limit=600)
def send_scheduled_reports():
    SendScheduledReports().run()
